%% ------- INTRODUCTION ------ %%
% Author: Natasha Birari (natashabirari@gmail.com)

% This script is used to analyse the PA power consumption model. 
% It needs the following scripts to be run first: main.m

% The section 'Theoretical' lists the assumptions followed and plots
% generated for the analysis.

%% +++++++++++++ THEORETICAL +++++++++++++++++++++++++++++
% This script has three sequential parts. The output of part 3 is the total
% power consumption of the DACs in our circuit, or P_DAC.
% 
% PART 1 depends on regulations on SFDR and basic design choices.
% Output of part 1 is the ENOB and NOB_min; they are inputs to part 2.

% PART 2 depends on the choice of f_sample and the part 1 output, and 
% varies with the bandwidth (i.e. use cases).
% Output of part 2 is the power consumption per DAC.
% Since we model using 3 different bandwidth values, there will be 3 output
% values of the DAC power consumption.

% PART 3 calculates total P_DAC depending on the output of part 2 and the
% number of DACs 'num_DACs' in an architecture. The number of DACs varies either 
% (1) with Ntx (2DA), where the number of users N_user is independent of 
%     num_DACs. There will be 'Nuser' output curves.
% (2) with M (minimum) = N_user (HSA, HFC, DMBA, MBPAA), where Ntx is 
%     independent of num_DACs. There will be 12 output curves since 
%     different use cases apply to different values of N_user. See the main
%     paper for the feasibility of use case and N_user.
%
% -----------------     Plots generated     ----------------- %
% Several plots for DAC power consumption analysis are generated in this
% script and were part of the main paper:
% Figs 1-3: parametric behaviour of P_DAC model
% 1. Effect of fs on P_DAC (for 3 different values of ENOB)
% 2. Effect of ENOB on P_DAC (for 3 different values of fs)
% 3. Effect of Vdd on P_DAC (for 3 different values of ENOB)
% Figs 4-5: surveyed data vs computed P_DAC
% 4. DAC model validation (computed/theoretical vs reported P_DAC) - error plots
% 5. Computed vs reported values of DAC power consumption - scatter plots

% -----------------     List of assumptions     -------------------------%

% 1.    Single carrier, so crest factor = 0.
% 2.    Assume DAC is operating at fullscale
% 3.    Aim to meet SFDR requirements, so choose appropriate freq range
% 4.    Ideal DAC: 0 harmonic distortion (THD), 0 analog noise, 
%                  0 clock jitter, 0LSB DNL (differential non-linearity)

% SFDR requirements from 3GPP TS 38.104 v17.5.0 (2022-03)
% for BS Tx FR2
% 1.    Freq range: 9kH --> 1 GHz offset from centre frequency
% 2.    measurement BW: 10 MHz
% 3.    Category B limit, i.e. more stringent e.g. in Europe

% DR requirements from ETSI report
% 1.    Headroom requirement more than for PA; latter is 6 dB PBO_out.
% 2.    DAC filters assumed to operate at an attenuation from centre freq.

% Technological assumptions:
% 1.    Only current-steering DACs considered.
% 2.    Formula in Cui 2005A is a first-order model, hence variations
%       between reported and modelled power consumption could be +- 50%
%       even without accounting for specific architectural peculiarities.
% 3.    For the DAC: Io, Cp and ___ are used as reported in Cui 2005A. See
%       part 3 for definitions of parameters and their values.
% 4.    Since the ENOB is often smaller than the reported resolution of a
%       DAC, in the formula in 2005A Cui, the number of bits 'n' is taken
%       to be 0.75 bits lower than the reported resolution in our survey.


%% ----------------- PART 1 ----------------- %%

SFDR_min = -36;                 % dBc, min SFDR for BS Tx FR2
HR = 10;                        % dB, headroom
filter_attenuation = 3;        % dB, attenuation from filters at offset

% Dynamic range of DAC --> equivalent to SINAD for ideal DAC
DR_DAC = -SFDR_min + HR - filter_attenuation;           % dB

% Oversampling factor 2x w.r.t. Nyquist rate
% Effect on SINAD is quantifiable and given by 10*log10(F_os/F_s) where F_s
% is Nyquist sampling rate and F_os is oversampling rate (from
% imp_dac_definitions_1)
OS_factor_DAC = 2;    % based on Yan (2019), and w.r.t fmax

% Based on above aspects, get SINAD (Signal to Noise and Distortion Ratio)
% 0 harmonic distortions within SFDR frequency range (see Assumptions)
SINAD = DR_DAC + 10*log10(OS_factor_DAC);    % dB, quantisation noise contained in HR

% Based on SINAD, find ENOB and NOB using the common formula; 
% 1.76 = 10*log10(3/2)
ENOB = (SINAD - 1.76)./6.02;            % Effective Number of Bits
NOB = ceil(ENOB);                       % minimum DAC resolution

%% ----------------- PART 2 ----------------- %%

BW_dac = bandwidths;

% commonly used value. Digital supply voltage not considered.
Vdd = 2.7;              % V, analog supply voltage

% f_cor = DAC corner frequency. Differentiate for our 3 use cases.
f_cor = [0.5 0.8 1]*1e9;        
OS_factor_dac = 2;

% Note on f_cor:
% In this work the f_cor max is 1 GHz, which is 2.5 times the highest BW in 
% f_signal.
% f_cor can be scaled by 10^3 compared to the values in Cui (2005) 
% because our bandwidths are scaled by an order of 10^4 compared to the 
% bandwidth in Cui (2005).
% In Cui (2005): Bandwidth = 1kHz, f_cor = 10kHz

% Theoretical power consumption for all bandwidths with computed (min) NOB
% input: OS_factor, Vdd (V), ENOB (bits), BW (Hz)
[P_per_dac_th_min_nob, fs_dac_range] = get_per_DAC_power(Vdd, NOB, BW_dac, f_cor, OS_factor_dac); % W
P_per_dac_th_min_nob = round(P_per_dac_th_min_nob*1000);    % mW

% for further analysis, since fs is BW related
fs_dac_range = fs_dac_range/1e9;                        % GHz

%% Calculate power using other resolution values, all in mW
[P_per_dac_th_10] = round(get_per_DAC_power(Vdd, 10, BW_dac, f_cor, OS_factor_dac)*1000);
[P_per_dac_th_12] = round(get_per_DAC_power(Vdd, 12, BW_dac, f_cor, OS_factor_dac)*1000);
[P_per_dac_th_14] = round(get_per_DAC_power(Vdd, 14, BW_dac, f_cor, OS_factor_dac)*1000);
[P_per_dac_th_16] = round(get_per_DAC_power(Vdd, 16, BW_dac, f_cor, OS_factor_dac)*1000);

P_per_dac_all = {P_per_dac_th_min_nob; P_per_dac_th_10; P_per_dac_th_12; ...
                 P_per_dac_th_14;  P_per_dac_th_16};

%% ---------------- PART 3 ----------------- %%

% Implemented directly in the main script.


%% +++++++++++++ IMPORTED SURVEY ++++++++++++++++++++++++++

% CMOS Current-Steering DACs
% RF and BB DACs, published papers and Analog Devices COM implementations.
% Years: 2 from before 2006, rest between 2010 - 2021
% All meet SFDR minimum limit i.e. SFDR magnitude is > 36 dBc.

% Parts 1 and 2 are integrated since the imported table has a column for
% Pdc_per_DAC (mW).

% Part 2 results of theoretical part and imported part are plotted together

T = readtable('..\Modelling\DAC_survey_preproc.csv');
T_units = readtable('..\Modelling\DAC_survey_units.csv');

%% Data processing and cleaning

% Convert technology and dac_type column to a categorical, rename CMOS SOI 
% for consistency
T.dac_type = categorical(T.dac_type);
T.technology = categorical(T.technology);
T.technology = renamecats(T.technology,'CMOS FDSOI','CMOS_SOI');

% Only choose current-steering (CS) DACs
T = T(find(strcmp(T.current_steering,'yes')),:);

% Add references: create extra column for numbering in the plot
ref = [1:1:size(T,1)]';
T = addvars(T, ref, 'Before', 1);

% Fill in NaN values in Vdd columns
T.vdd_digital(isnan(T.vdd_digital)) = mean(T.vdd_digital(~isnan(T.vdd_digital)));  % replace N.A. values with mean value
T.vdd_analog(isnan(T.vdd_analog)) = mean(T.vdd_analog(~isnan(T.vdd_analog)));  % replace N.A. values with mean value

%% ----------------- PLOTS ----------------- %%
% The sections below plot the power consumption per DAC (Pdc_per_DAC) 

%% Figures 1-3: Paramteric behaviour of P_DAC model

% Choose use case to plot
toggle_case_num = 2;  % case number 1, 2 or 3

% Run this cell once to generate figures 1-3 for a given use case
for run_once = 1 
%% Figure 2: Effect of fs on P_DAC (grouped by ENOB)
% Since fs is a combination of (f_cor, BW), there is no further 
% differentiation between use cases with fs, as 
% both f_cor and BW change in tandem.

fprintf('-------- Testing effect of fs: case %d --------\n',toggle_case_num);

% Choose bandwidth and corresponding f_cor based on use case
t_bw = bandwidths(toggle_case_num);              % Hz
f_cor_temp = f_cor(toggle_case_num);            % Hz

% sampling: 2x = Nyquist rate, 4x = OS max.
os_temp = 1:0.1:2;                              % oversampling factor

% Define x axis
t_fs = 2*(f_cor_temp + t_bw/2)*os_temp;         % Hz

% Choose 3 values of ENOB to plot
enob_temp = [8 12 16];

% Get power consumption per DAC, in W
pdc1fs = get_per_DAC_power(Vdd, enob_temp(1), t_bw, f_cor_temp, os_temp);
pdc2fs = get_per_DAC_power(Vdd, enob_temp(2), t_bw, f_cor_temp, os_temp);
pdc3fs = get_per_DAC_power(Vdd, enob_temp(3), t_bw, f_cor_temp, os_temp);

% Plot 
figure; plot(t_fs/1e9, round(pdc1fs*1000), t_fs/1e9, round(pdc2fs*1000), t_fs/1e9, round(pdc3fs*1000), 'LineWidth',1);
legend('8 bits','12 bits','16 bits', 'Location','NW');
xlabel('f_s [GHz]'); ylabel('P_{DAC} [mW]');
title({"Varying f_s, fixed ENOB", "Case "+toggle_case_num});
%xlim([t_fs(1),t_fs(end)]./1e6); 
grid on;
 
%% Figure 3: Effect of ENOB (grouped by f_cor)

fprintf('-------- Testing effect of ENOB: case %d --------\n',toggle_case_num);

% Choose bandwidth and corresponding f_cor based on use case
t_bw = bandwidths(toggle_case_num);                 % Hz
f_cor_temp = f_cor(toggle_case_num);                % Hz

% sampling: 2x = Nyquist rate, 4x = OS max.
% Choose 3 different values of OS rate to plot
os_temp = [1 1.5 2];

% Define x axis
t_enob = 8:1:16;                                    % bits

% Get power consumption per DAC, in W
pdc1e = get_per_DAC_power(Vdd,t_enob,t_bw,f_cor_temp,os_temp(1));
pdc2e = get_per_DAC_power(Vdd,t_enob,t_bw,f_cor_temp,os_temp(2));
pdc3e = get_per_DAC_power(Vdd,t_enob,t_bw,f_cor_temp,os_temp(3));

% Plot
figure; plot(t_enob, round(pdc1e*1000), t_enob, round(pdc2e*1000), t_enob, round(pdc3e*1000),'LineWidth',1);
legend('OS x1','OS x1.5','OS x2', 'Location','NW');
xlabel('ENOB [bits]'); ylabel('P_{DAC} [mW]');
title({"Varying ENOB, fixed BW and fixed f_{cor}","Case "+toggle_case_num});
xlim([t_enob(1),t_enob(end)]); grid on;
 
%% Figure 4: Effect of Supply Voltage Vdd (grouped by ENOB)

fprintf('-------- Testing effect of Vdd: case %d --------\n',toggle_case_num);

% Choose bandwidth and corresponding f_cor based on use case
t_bw = bandwidths(toggle_case_num);             % Hz
f_cor_temp = f_cor(toggle_case_num);           % Hz

% sampling: 2x = Nyquist rate, 4x = OS max.
os_temp = 2;                                    % fixed

% Define x axis
t_vdd = 1:0.1:3.3;                              % V

% Choose 3 values of ENOB to plot
enob_temp = [8 12 16];

% Get power consumption per DAC, in W
pdc1v = get_per_DAC_power(t_vdd, enob_temp(1), t_bw, f_cor_temp, os_temp);
pdc2v = get_per_DAC_power(t_vdd, enob_temp(2), t_bw, f_cor_temp, os_temp);
pdc3v = get_per_DAC_power(t_vdd, enob_temp(3), t_bw, f_cor_temp, os_temp);

% Plot
figure; plot(t_vdd, round(pdc1v*1000), t_vdd, round(pdc2v*1000), t_vdd, round(pdc3v*1000),'LineWidth',1);
legend('8 bits','12 bits','16 bits', 'Location','NW');
xlabel('V_{dd} [V]'); ylabel('P_{DAC} [mW]');
title({"Varying V_{dd}, fixed ENOB and fixed f_s", "Case "+toggle_case_num});
xlim([t_vdd(1),t_vdd(end)]); grid on;
 
end

% Clear temporary variables
clear ('toggle_case_num','t_bw','f_cor_temp','os_temp', 't_enob', 't_vdd', 'run_once');

%% Figure 4: DAC model validation 
% i.e. error checking: theoretical vs reported values

% Checks error between reported and computed P_dc values by plugging in
% reported specs from the DAC survey into the model used in this work

t_pow_reported = T.pow_consumption;             % mW

t_vdd = T.vdd_analog;                           % V
t_enob = T.resolution - 0.75;                   % bits
t_process = T.process;                          % nm, not used in model.

% Reported P_dc was usually measured at reported dampling frequency fs, so
% use the same reported fs as input for computed value
t_fs = T.f_sample*1e9;                          % Hz
t_fs_title = "reported";

% DAC internal params from funtion: get_per_DAC_power()    
t_Io = 10e-6;   % 10 uA
t_Cp = 1e-12;   % 1 pF
t_beta = 1;

% Use formula to get computed value
t_pow_comp = t_beta*(0.5*t_Io.*t_vdd.*(2.^(t_enob) - 1) + 0.5*t_enob.*t_Cp.*(t_fs).*t_vdd.^2); % W
t_pow_comp = t_pow_comp*1000;                                       % mW 

% Geometry (process) dependent power consumption - needs further analysis

% Calculate errors
err1 = t_pow_reported - t_pow_comp;                         % absolute error 
err2 = (t_pow_reported - t_pow_comp)./t_pow_reported*100;   % percentage error
refers_old = cellstr(num2str(T.ref));                        % store references

% statistics --> exclude the 8th datapoint (see main paper)
mu1 = round(mean(err1([1:7,9:end])),2);
mu2 = round(mean(err2([1:7,9:end])),2);
std2 = round(std(err2([1:7,9:end])),2);
std1 = round(std(err1([1:7,9:end])),2);

% Turn reference to string
refers = cellstr(num2str([1:1:12]'));

% Plot
for makefig = 1
    figure; 
    % ..... Plot absolute error .....
    subplot(1,2,1); title('Absolute error');
    hold on;
    plot(1:1:size(T,1), err1, 'k-o');
    % % -------- Optional: color the data points where Vdd > 2.5 V ----------
    % temp_idx = table2array(T(T.vdd_analog > 2.5, 'ref'));           % get index
    % ghs = scatter(temp_idx, err1(temp_idx),40,'Marker', 'o', 'MarkerFaceColor', 'none',...
    %    'MarkerEdgeColor','b', 'LineWidth',0.8, 'DisplayName','VddA > 2.5V');
    % legend(ghs, 'Location','SW');
    % %  ---------------------------------------------------------------------
    hold off;
    xticks(1:1:size(T,1)); ylabel('Error (mW)','FontSize',12);
    xticklabels(refers); xlabel('Reference');
    box on;
    % ..... Plot percentage error .....
    subplot(1,2,2); title('% error');
    hold on;
    plot(1:1:size(T,1), err2, 'k-o');
    % % -------- Optional: color the data points where Vdd > 2.5 V ----------
    % ghs = scatter(temp_idx, err2(temp_idx),40,'Marker', 'o', 'MarkerFaceColor', 'none',...
    %    'MarkerEdgeColor','b', 'LineWidth',0.8, 'DisplayName','VddA > 2.5V');
    % text(1:1:size(T,1), err2+1, num2str(T.resolution));
    %  legend(ghs, 'Location','SW');
    % %  ---------------------------------------------------------------------
    hold off; 
    xticks(1:1:size(T,1)); ylabel('Error (%)','FontSize',12);
    xticklabels(refers); xlabel('Reference');
    box on;
    % .... common title ....
    sgtitle({"I_o = "+t_Io*1e6+"uA, C_p = "+t_Cp*1e12+"pF, \beta = "+t_beta+", f_s = "+t_fs_title+" GHz", ...
        "\epsilon_{mean}= "+mu2+"%, \epsilon_{\sigma} = "+std2+"% = "+std1+"mW", ...
        "(statistics excluding Ref. 8)"});

    % Clear temporary variables
    clear ('t_bw','t_fs','t_fs_title', 't_enob','t_vdd','t_Io','t_Cp','t_beta');
end

%% Figure 5: Plot computed vs reported power consumptions
% The plotting cells below can be run independently to generate each plot

mycmap = colormap(parula(7));        % Choose color map 
T_val = T;                           % create a new table to edit freely

% divide sampling frequency ranges into groups
fsg = [];
for i = 1:size(T_val,1)
   fval =  T_val.f_sample(i);
   if fval <= 1                         % less than 1 GHz (fewer DACs in this range)
       fsg(i) = 1;
   elseif fval > 1 && fval <= 2         % between 1-2 GHz
       fsg(i) = 2;
   elseif fval > 2 && fval <= 3         % between 2-3 GHz
       fsg(i) = 3;
    elseif fval > 3 && fval <= 4        % between 3-4 GHz
        fsg(i) = 4;
    elseif fval > 4 && fval <= 8        % between 4-8 GHz
        fsg(i) = 8;
    elseif fval > 8 && fval <= 12       % between 8-12 GHz 
        fsg(i) = 12;
   else
       fsg(i) = 16;                     % higher than 12 GHz (fewer DACs in this range)
   end    
end
fsg = fsg';

% add fs groups to table
myLegendDetails = {'<1','1-2','2-3','3-4','4-8','8-12','12<'};
T_val = addvars(T_val, fsg, 'After','f_sample');

% add t_pow_comp to table
% IMPORTANT: check that it was computed using fs = 'reported value' in previous code block
% to ensure the same inputs while comparing reported vs computed outputs
T_val = addvars(T_val, t_pow_comp, 'After','pow_consumption');

%% Plot Reported values: Fig 5_1 and Fig 5_2
%% Figure 5_1: Pdc vs ENOB, grouped by f_sample (colour groups).
% Run cell to generate plot

for makefig = 1
figure; xlabel('resolution (bits)'); ylabel('DAC power consumption (mW)');
grid on; hold on;
% for SFDR > 70 dBc ......
ghs1 = scatter(table2array(T_val(T_val.SFDR > 70, 'resolution')),...
    table2array(T_val(T_val.SFDR > 70, 'pow_consumption')), ...
    100,'MarkerEdgeColor', 'k','Marker', '+', 'LineWidth',1,...
    'DisplayName','SFDR > 70dBc'); 
% for different sampling frequency groups ....
ghs2 = gscatter(T_val.resolution, T_val.pow_consumption, T_val.fsg,mycmap, [],20);
% Set coordinates for plotting references
dx = 0.1; dy = 0.1; 
refers = cellstr(num2str(T_val.ref));   % add refs to points
dactext = cellstr(T_val.dac_type);      % add dac_type to points
text(T_val.resolution+dx, T_val.pow_consumption+dy, refers+","+dactext,'FontSize',8); 
% Set legends and axis properties
ax1=gca; % uncomment when plotting ghs1
ax2=axes('position',get(gca,'position'),'visible','off'); % uncomment when plotting ghs4
leg1 = legend(ax1, ghs1, 'Location','NorthEastOutside'); % uncomment when plotting ghs1
leg2 = legend(ax2, ghs2, myLegendDetails{1:7}, 'Location','EastOutside'); % uncomment when plotting ghs2
title(leg2, 'f_s (GHz)');
hold off;
title(ax1,'Reported');
ylim(ax1, [0 1000]);
box(ax1,'on');
end

%% Figure 5_2: Pdc vs f_sample, grouped by ENOB (colour groups)
% Run cell to generate plot

for makefig = 1
figure; xlabel('fs (Ghz)');ylabel('DAC power consumption (mW)');
grid on; hold on;
ghs1 = scatter(table2array(T_val(T_val.SFDR > 70, 'f_sample')),...
    table2array(T_val(T_val.SFDR > 70, 'pow_consumption')), ...
    100,'MarkerEdgeColor', 'k','Marker', '+', 'LineWidth',1,...
    'DisplayName','SFDR > 70dBc'); 
ghs2 = gscatter(T_val.f_sample, T_val.pow_consumption, T_val.resolution,colormap(parula(4)), [],20);
hold off;
dx = 0.1; dy = 0.1; 
refers = cellstr(num2str(T_val.ref));      % add refs to points
dactext = cellstr(T_val.dac_type);      % add dac_type to points
text(T_val.f_sample+dx, T_val.pow_consumption+dy, refers+","+dactext,'FontSize',8); 
% Set legends and axis properties
ax1=gca;
ax2=axes('position',get(gca,'position'),'visible','off');
leg1 = legend(ax1, ghs1, 'Location','NorthEastOutside');
leg2 = legend(ax2, ghs2, 'Location','EastOutside');
title(leg2, 'ENOB');
title(ax1,'Reported'); ylim(ax1, [0 1000]);
end

%% Plot Computed values: Fig 5_3 and Fig 5_4
%% Figure 5_3: Pdc vs ENOB, grouped by f_sample (colour groups)
% Run cell to generate plot

for makefig = 1
figure; xlabel('resolution (bits)');ylabel('DAC power consumption (mW)');
grid on; hold on;
ghs1 = scatter(table2array(T_val(T_val.SFDR > 70, 'resolution')),...
    table2array(T_val(T_val.SFDR > 70, 't_pow_comp')), ...
    100,'MarkerEdgeColor', 'k','Marker', '+', 'LineWidth',1,...
    'DisplayName','SFDR > 70dBc'); 
ghs2 = gscatter(T_val.resolution, T_val.t_pow_comp, T_val.fsg,mycmap, [],20);
dx = 0.1; dy = 0.1; 
refers = cellstr(num2str(T_val.ref));      % add refs to points
dactext = cellstr(T_val.dac_type);      % add dac_type to points
text(T_val.resolution+dx, T_val.t_pow_comp+dy, refers+","+dactext,'FontSize',8); 
% Set legends and axis properties
ax1=gca;
ax2=axes('position',get(gca,'position'),'visible','off');
leg1 = legend(ax1, ghs1, 'Location','NorthEastOutside');
leg2 = legend(ax2, ghs2, myLegendDetails{1:7}, 'Location','EastOutside');
title(leg2, 'f_s (GHz)');
hold off;
title(ax1,'Computed');
box(ax1,'on');
end

%% Figure 5_4: Pdc vs f_sample, grouped by ENOB (colour groups)
% Run cell to generate plot

for makefig = 1
figure; xlabel('fs (Ghz)');ylabel('DAC power consumption (mW)');
grid on; hold on;
ghs1 = scatter(table2array(T_val(T_val.SFDR > 70, 'f_sample')),...
    table2array(T_val(T_val.SFDR > 70, 't_pow_comp')), ...
    100,'MarkerEdgeColor', 'k','Marker', '+', 'LineWidth',1,...
    'DisplayName','SFDR > 70dBc'); 
ghs2 = gscatter(T_val.f_sample, T_val.t_pow_comp, T_val.resolution,colormap(parula(4)), [],20);
% Set coordinates for plotting references
dx = 0.1; dy = 0.1; 
refers = cellstr(num2str(T_val.ref));      % add refs to points
dactext = cellstr(T_val.dac_type);      % add dac_type to points
text(T_val.f_sample+dx, T_val.t_pow_comp+dy, refers+","+dactext,'FontSize',8); 
% Set legends and axis properties
ax1=gca;
ax2=axes('position',get(gca,'position'),'visible','off');
leg1 = legend(ax1, ghs1, 'Location','NorthEastOutside');
leg2 = legend(ax2, ghs2, 'Location','EastOutside');
title(leg2, 'ENOB');
hold off;
title(ax1,'Computed');
end

% Clear temporary value
clear('makefig');