%% ------- INTRODUCTION ------ %%
% Author: Natasha Birari (natashabirari@gmail.com), Aug 2022

% This script is used to analyse the PA power consumption model. 
% It needs the following scripts to be run first: 
% main_model_fullyconnected.m
% main_model_subarray.m

%% +++++++++++++++++ THEORETICAL +++++++++++++++++

% _____PA_____:
% get the required output power per PA for 2 cases:
%   (1) as many PAs as antenna elements         num_PA = Ntx    default
%   (2) as many PAs as sub-arrays               num_PA = M      SDMBA architecture only
% plot the output power curves, and overlay with PA survey information.

% IMP NOTE: Currently, EIRP1 is fixed hence the output power is use-case
% (bandwidth) independent 

% ---------------    List of Assumptions --------------- %
% PAE_pbo is the same as PAE_avg from the PA survey. 
%
% The PAE_avg is either the 6db backoff PAE, or any other measure of 
% 'average' where graphical estimation was not possible.
%

%% Set up common plot settings
rng(1);
fontsize = 10; 

% for output power curves with different N_user
linVec = {'-','-','-','-','-','-'}; 
markerVec = {'o','*','d','^','s','x'}; % six markers

% Define colours
myColors = [        0,      0.4470, 0.7410;         0.8500, 0.3250, 0.098;
                    0.9290, 0.6940, 0.1250;         0.4940, 0.1840, 0.5560;
                    0.4660, 0.6740, 0.1880;         0,      0,      0];
        
myColorsTransp = [  0,      0.4470, 0.7410, 0.5;    0.8500, 0.3250, 0.098, 0.5;
                    0.9290, 0.6940, 0.1250, 0.5;    0.4940, 0.1840, 0.5560, 0.5;
                    0.4660, 0.6740, 0.1880, 0.5;    0, 0, 0, 0.5];

%% +++++++++++++++++ IMPORT DATA +++++++++++++++++

T = readtable('supporting_files/PA_survey_preproc.csv');
T_units = readtable('supporting_files/PA_survey_units.csv');

% Convert technology column to a categorical,merge CMOS_SOI rows, reorder
T.technology = categorical(T.technology);
T.technology = mergecats(T.technology, {'CMOS_SOI','CMOS_FD_SOI'});
T.technology = reordercats(T.technology, {'CMOS_Bulk','CMOS_SOI','SiGe','GaAs','GaN'});

% get types of PA techs as cell array
PA_techs = string(table2array(unique(T(:,'technology'))));
num_PA_techs = length(PA_techs);

% get only exactly 28 GHz data 
T = T(T.frequency == 28,:);

% convert PAE columns from % to decimal
if T.PAE_max > 1 | isnan(T.PAE_max)
    T.PAE_max = T.PAE_max/100;
end
if T.PAE_1dB > 1 | isnan(T.PAE_1dB)
    T.PAE_1dB = T.PAE_1dB/100;
end
if T.PAE_avg > 1 | isnan(T.PAE_avg)
    T.PAE_avg = T.PAE_avg/100;
end

% Only keep entries with Gain >= 13 dB
T = T(T.gain >= 13, :);

%% Compute gamma (factor) and add column
% create new column with gamma = (1-1/G)/PAE_avg, with G in linear scale

% Create new table for freely editing 
T_pdcf = T;

% Calculate gamma - a factor for PA Pdc 
gamma = (1 - 1./(db2pow(T_pdcf.gain)))./(T_pdcf.PAE_avg);   
T_pdcf = addvars(T_pdcf, gamma, 'Before', 'gain');

% create new 1st column for numbering references in the plot
ref = [1:1:size(T_pdcf,1)]';
T_pdcf = addvars(T_pdcf, ref, 'Before', 1);

% Export table T_pdcf
writetable(T_pdcf, 'T_pdcf');

%% Compute output power per PA for all architectures, N_user users

EIRP_temp = 40;

% preallocate
pout_1pa_fc = [];       % Fully connected (MBPAA / HFC) architecture
pout_1pa_hsa = [];      % hybrid subarray (HSA) architecture
pout_1pa_sdmba = [];      % subarray-based digital array (SDMBA) architecture
% DMBA does not need pre-allocation

% set temporary values of architecture and subarray size
temp_vals = ["MBPAA"];
K_range_temp = K_range;

for i = 1:length(N_user_range)
    N_user_temp = N_user_range(i);
    
    % ---- get output power for 1 PA for MBPAA architecture ----
    
    % dimensions of output matrix: x y = N_user Ntx
    [pout_1pa_fc(i,:),~] = get_per_PA_op(EIRP_temp, Gtx_antenna, N_user_temp, Ntx_range, temp_vals);

    % ---- get output power for 1 PA for subarray architectures (HSA, SDMBA) ----
    
    % set up subarray configurations for given number of users, number of
    % antenna elements, and subarray size
    for j = 1:length(K_range_temp)
        [sa_mat, ntx_mat] = create_subarrays(N_user_temp, Ntx_range,K_range_temp);
        
        if i == 4
            savethis = ntx_mat;
        end
        
        % dimensions of output matrix: x y z = N_user Ntx K
        [pout_1pa_hsa(i,:,j),~] = get_per_PA_op(EIRP_temp, Gtx_antenna, N_user_temp, ntx_mat(j,:), ["HSA", K_range_temp(j)]);
        [pout_1pa_sdmba(i,:,j),fidxt(i,:,j)] = get_per_PA_op(EIRP_temp, Gtx_antenna, N_user_temp, ntx_mat(j,:), ["SDMBA", K_range_temp(j)]);
    end
end

% Remove temporary variables
clear('temp_vals');

%% Plotting
%% (Fig A) Plot output power curves only
% Not included in the main paper

% Note : In some entries, PAE_avg was not available. In such cases Psat can still
% be plotted, but those data points do not contribute to the computation of
% average PAE for that technology; same for the avg Gain. 

% Get PA tech names from table
PA_tech_names = PA_techs;

for makefig = 1
    
% Fully connected architecture (MBPAA/HFC)
figure; hold on;
for x = 1:length(N_user_range) % loop over number of users
    plot(Ntx_range, pout_1pa_fc(x,:),'k-o', 'LineStyle',linVec{x},...
        'Marker', markerVec{x}, 'MarkerFaceColor','none','DisplayName',num2str(N_user_range(x)));
end
hold off; leg = legend; title(leg, 'Num. users'); 
grid on;
ylim([0 42]);
%xticks(Ntx_range); xticklabels({'16','32','64','128','256','512'}); 
xlabel('N_{TX}'); ylabel('P_{1dB_{PA}} (dBm)');
title({'PA output power required vs. number of Tx antennas','MBPAA, 2DA, HFC'});
box on;
%%
% Hybrid subarray architecture (HSA)
figure; hold on;
for r = 1:num_configs   % loop over number of subarray configurations
    subplot(2,3,r); hold on;
    for x = 1:length(N_user_range) % loop over number of users
        plot(Ntx_range, pout_1pa_hsa(x,:, r), 'k-o','LineStyle',linVec{x},...
            'Marker', markerVec{x}, 'DisplayName',num2str(N_user_range(x)));
    end
    hold off; grid on;
    xlabel('N_{TX}'); ylabel('P_{out} required (dBm)');
    title("K= "+K_range(r)); 
    ylim([-10,40]); 
    xlim([0, 600]);
    xticks([100 200 300 400 500 600]);
    %xticks(Ntx_range); xticklabels({'16','32','64','128','256','512'}); 
end
leg = legend('Location','best'); title(leg, 'Num. users');

sgtitle({'PA output power required vs. number of Tx antennas','HSA with K elements per sub-array'});
%%
% subarray-based digital array (SDMBA) architecture
figure; 
for r = 1:num_configs % loop over number of subarray configurations
    subplot(2,3,r); hold on;
    for x = 1:length(N_user_range) % loop over number of users
        plot(Ntx_range, pout_1pa_sdmba(x,:,r),'k-o','LineStyle',linVec{x},... 
            'Marker', markerVec{x}, 'DisplayName',num2str(N_user_range(x)));
    end
    hold off; grid on;
    xlabel('N_{TX}'); ylabel('P_{out} required (dBm)');
    title("K= "+K_range(r)); 
    ylim([-10,50]); 
    xlim([0, 600]);
    xticks([100 200 300 400 500 600]);
end
leg = legend('Location','best'); title(leg, 'Num. users');
sgtitle({'PA output power required vs. number of Tx antennas','SDMBA with K elements per sub-array'});

end

clear('makefig');

%% (Fig B) Compare PA technologies and properties 
% for required PA output power P_out, array size Ntx
% Not included in the main paper

% Draw figure:
% Along Y-left X-bottom: required Pout_PA vs Ntx
% Along Y-right X-top: Psat vs gamma

for i = 1:num_PA_techs                    % subfig for each tech    
    figure; t = tiledlayout(1,1);
    % plot 1
    ax1 = axes(t);                                  % for YLXB
    hold on;
    for x = 1:length(N_user_range)              % plot output power curves
        ghs1(x) = plot(Ntx_range, pout_1pa_fc(x,:),'k-','LineStyle', ...
            linVec{x},'Marker',markerVec{x});
        %text(530, pout_1pa_fc(x,end), num2str(N_user_range(x)));   % print N_user value
    end
    %text(520, pout_1pa_fc(5,end)+4,"N_{user}");        % N_user header
    ax1.XColor = 'b'; ax1.YColor = 'b';                     % set axis1 colours
    ylabel('P_{out} required (dBm)'); xlabel('N_{TX}');     % set axis1 labels
    yl = [0, 40];                                              % get axis1 Y-limits
    ylim(yl); 
    % plot 2
    ax2 = axes(t);                                  % for YRXT
    % save relevant columns in tempData 
    tData = table(table2array(T_pdcf(T_pdcf.technology == PA_techs(i),'ref')),...
        table2array(T_pdcf(T_pdcf.technology == PA_techs(i),'gamma')), ...
        table2array(T_pdcf(T_pdcf.technology == PA_techs(i),'Psat')), ... 
        'VariableNames',{'ref','gamma','Psat'});
    ghs = scatter(tData.gamma, tData.Psat, 20, 'r', 'filled');
    % DISPLAY OPTION: To insert Psat lines where pdc_factor is #Nan,
    % comment out the line below
        hData = table2array(tData(~isnan(tData.gamma),'Psat'));  
    for y = 1:size(hData,1)                     % loop over tData to plot ylines (Psat contours)
        yline(hData(y), 'Color','r','LineWidth',1);
    end  
    dx = 0.1; dy = 0.2; 
    refers = cellstr(num2str(tData.ref));
    if i == 3
        text(tData.gamma(1), tData.Psat(1)+7*dy, refers(1),'FontSize',8);  % avoid overlap for sige plot specifically
        text(tData.gamma(4), tData.Psat(4)-7*dy, refers(4),'FontSize',8);  % avoid overlap for sige plot specifically
        text(tData.gamma([2,3,5])+dx, tData.Psat([2,3,5])+dy, refers([2,3,5]),'FontSize',8); 
    else
        text(tData.gamma+dx, tData.Psat+dy, refers,'FontSize',8);
    end
    ylabel('P_{sat} (dBm)'); xlabel('\gamma', 'FontWeight','bold');    % set axis2 labels
    ax2.XAxisLocation = 'top'; ax2.YAxisLocation = 'right';
    ax2.Color = 'none';
    hold off;
    ylim(yl); xlim([0,30])                                  % set same Y-limits as axis1 (Pout and Psat)
    ax1.Box = 'off'; ax2.Box = 'off';
    grid on;
    %title({'PA properties and required PA output power','for varying array size N_{TX} and number of users N_{USER}'});
    %title(PA_techs(i)+" PA properties vs. required PA output",'interpreter','none');
    legend(PA_techs(i),'interpreter','none','Location','SW');
    set(gcf,'units','centimeters','position',[10,5,15,12]);
    figname = ['saved matlab figures/component_figs/PA_techs' num2str(i) 'sub'];
    saveas(gcf,figname,'png');
    %saveas(gcf,figname,'fig');
    
    % Add legend of N_user: superimpose on ax2 to avoid gridlines
    leg1 = legend(ax1, ghs1, num2str(N_user_range'));    title(leg1,'Num. users');
end

%% Survey Analysis: average values of relevant PA parameters
% PA parameters were analysed with respect to their inter-relationships and
% Relevant parameter values were averaged across the surveyed PAs
% (differentiating by PA technology) to use in the selection and further
% analysis of the PA.
% These averaged properties were saved to 'PA_specifications_thiswork.txt'

% Create copy of table
T_cpy = T_pdcf;
% T_cpy = T_pdcf(T_pdcf.gain >= 13,:); % Only PAs with high gain

% Add new columns
tmp = NaN*ones(size(T_cpy,1),1);
T_cpy = addvars(T_cpy, tmp, tmp, tmp, tmp, 'NewVariableNames',...
    {'avg_PAE_avg', 'avg_PAE_max','avg_Psat','avg_gain'}, ...
    'After','frequency');
tmp = NaN*ones(5,1);
survey_stats = table(tmp,tmp,tmp,tmp,tmp,tmp,tmp,tmp,tmp,tmp, 'VariableNames',...
    ["min_Psat" "max_Psat" "mean_Psat" "min_G" "max_G" "mean_G" "min_PAE_avg" "max_PAE_avg" "mean_PAE_avg" "mean_PAE_max"]) ;
survey_stats = addvars(survey_stats, PA_techs, 'NewVariableName','technology', 'Before',1);

%% Get average, min and max values of relevant properties per PA technology
% These values have been saved in the file 
% 'supporting_files/PA_specifications_thiswork.txt'

% Psat: min, max, mean
store_val = {};        
for i = 1:5
   dat = table2array(T_cpy(T_cpy.technology == PA_tech_names(i), 'Psat'));
   sz = size(dat,1);
   mu = mean(dat, 'omitnan');
   store_val{i} = repmat(mu,[1,sz])';
   T_cpy(T_cpy.technology == PA_tech_names(i), 'avg_Psat') = table(store_val{i});
   survey_stats{survey_stats.technology == PA_tech_names(i), 'min_Psat'} = min(dat);
   survey_stats{survey_stats.technology == PA_tech_names(i), 'max_Psat'} = max(dat);
   survey_stats{survey_stats.technology == PA_tech_names(i), 'mean_Psat'} = mu;
end
clear('dat','store_val','mu','sz');

% Gain: min, max, mean
store_val = {};        
for i = 1:5
   dat = table2array(T_cpy(T_cpy.technology == PA_tech_names(i), 'gain'));
   sz = size(dat,1);
   mu = mean(dat, 'omitnan');
   store_val{i} = repmat(mu,[1,sz])';
   T_cpy(T_cpy.technology == PA_tech_names(i), 'avg_gain') = table(store_val{i});
   survey_stats{survey_stats.technology == PA_tech_names(i), 'min_G'} = min(dat);
   survey_stats{survey_stats.technology == PA_tech_names(i), 'max_G'} = max(dat);
   survey_stats{survey_stats.technology == PA_tech_names(i), 'mean_G'} = mu;
end
clear('dat','store_val','mu','sz');

% PAE_avg: min, max, mean
store_val = {};        
for i = 1:5
   dat = table2array(T_cpy(T_cpy.technology == PA_tech_names(i), 'PAE_avg'));
   sz = size(dat,1);
   mu = mean(dat, 'omitnan');
   store_val{i} = repmat(mu,[1,sz])';
   T_cpy(T_cpy.technology == PA_tech_names(i), 'avg_PAE_avg') = table(store_val{i});
   survey_stats{survey_stats.technology == PA_tech_names(i), 'min_PAE_avg'} = min(dat);
   survey_stats{survey_stats.technology == PA_tech_names(i), 'max_PAE_avg'} = max(dat);
   survey_stats{survey_stats.technology == PA_tech_names(i), 'mean_PAE_avg'} = mu;

end
clear('dat','store_val','mu','sz');

% PAE_max: mean
store_val = {};        
for i = 1:5
   dat = table2array(T_cpy(T_cpy.technology == PA_tech_names(i), 'PAE_max'));
   sz = size(dat,1);
   mu = mean(dat, 'omitnan');
   store_val{i} = repmat(mu,[1,sz])';
  survey_stats{survey_stats.technology == PA_tech_names(i), 'mean_PAE_max'} = mu;
end
clear('dat','store_val','mu','sz');

vec_one_val_all_techs = [1,8,15,27,32];         % MANUALLY CHOSEN FOR PLOTS

% Get average value of gamma
avg_gamma = (1-1./T_cpy.avg_gain(vec_one_val_all_techs))./(T_cpy.avg_PAE_avg(vec_one_val_all_techs));
survey_stats = addvars(survey_stats, avg_gamma, 'NewVariableName','avg_Gamma');

% round all entries to 2 places
temp_tab = varfun(@(x) round(x, 2), survey_stats, 'InputVariables', @isnumeric);
temp_tab = addvars(temp_tab, survey_stats.technology, 'Before',1);
temp_tab.Properties.VariableNames = survey_stats.Properties.VariableNames;
survey_stats = temp_tab; clear('temp_tab');

%% (Fig C) Plot survey_stats - scatter plots of Psat vs average PAE (min, mean and max)
% Not included in main paper

for makefig = 1
figure; hold on; 
subplot(1,3,1); gscatter(survey_stats.min_PAE_avg*100, survey_stats.min_Psat, survey_stats.technology, [], [], 25);
xlabel('PAE_{avg}%'); ylabel('Psat (dBm)'); title('min'); grid on; %ylim([0 30]);
subplot(1,3,2); gscatter(survey_stats.mean_PAE_avg*100, survey_stats.mean_Psat, survey_stats.technology, [], [],25);
xlabel('PAE_{avg}%'); ylabel('Psat (dBm)'); title('mean'); grid on; %ylim([0 30]);
subplot(1,3,3); gscatter(survey_stats.max_PAE_avg*100, survey_stats.max_Psat, survey_stats.technology, [], [], 25);
xlabel('PAE_{avg}%'); ylabel('Psat (dBm)'); title('max'); grid on; %ylim([0 30]);
xlabel('PAE_{avg}%'); ylabel('Psat (dBm)'); %ylim([0 30]);
sgtitle('Min, Avg and Max values per PA technology in the state-of-the-art');
grid on; legend;
end

%% (D) Average Psat per technology vs. required P_out for each archicture
% Included in main paper

% Set common plot settings
cmap = cool(5);
yl1 = [0 42];
yl2 = [-10 45];

% List of figures generated:
% Fig D_1: fully connected architectures: P_out (max) vs. mean P_sat
% Fig D_2: fully connected architectures: P_out (at PBO) vs. mean P_sat
% Fig D_3: SDMBA architecture: P_out (max) vs. mean P_sat
% Fig D_4: SDMBA architecture: P_out (at PBO) vs. mean P_sat
% Fig D_5: HSA architecture: P_out (max) vs. mean P_sat
% Fig D_6: HSA architecture: P_out (at PBO) vs. mean P_sat

% NOTE:
% Operational P_out values at are calculated at the specified 6dB PBO.
% However, the max P_out is used to find the suitable PA techs for a given
% scenario and set of inputs.

%% Figure D_1: fully connected architectures: Required P_out (max) vs. mean P_sat for each technology
% for MBPAA, HFC and DMBA

for makefig = 1
figure; grid on; yyaxis left; hold on;
for i = 1:length(N_user_range)
   ghs1(i) = plot(Ntx_range, pout_1pa_fc(i,:),'k-o', 'LineStyle',linVec{i},...
   'Marker', markerVec{i}, 'MarkerFaceColor','none','DisplayName',num2str(N_user_range(i)));
end
hold off; ax1 = gca; ax1.YColor = 'black'; colororder('black'); 
ylim(yl1);
 xlabel('N_{TX}'); ylabel('P_{1dB_{PA}}, mean P_{sat} (dBm)');
yyaxis right;  hold on;
for i = 1:length(survey_stats.technology)
   ghs2(i) = yline(survey_stats.mean_Psat(i), 'Color', myColors(i,:), ...
   'LineWidth',2,'DisplayName',string(survey_stats.technology(i))); 
end
hold off; ax2 = gca; ax2.YColor = 'black';
ylim(yl1); % yticks([]); 
box on;
leg = legend('NumColumns',2,'Location','eastoutside', 'interpreter','none'); title(leg, 'Num. users            PA tech.');
end
sgtitle({"Required P_{1dB_{PA}} vs. mean P_{sat} - MBPAA, DMBA, HFC,"," EIRP "+EIRP_temp+"dBm"});

%% Figure D_2: fully connected architectures: Required P_out at 6dB PBO vs. mean P_sat for each technology
% for MBPAA, HFC and DMBA

for makefig = 1
figure; grid on; yyaxis left; hold on;
for i = 1:length(N_user_range)
   ghs1(i) = plot(Ntx_range, pout_1pa_fc(i,:) - 6,'k-o', 'LineStyle',linVec{i},...
   'Marker', markerVec{i}, 'MarkerFaceColor','none','DisplayName',num2str(N_user_range(i)));
end
hold off; ax1 = gca; ax1.YColor = 'black'; colororder('black'); 
ylim(yl2);
 xlabel('N_{TX}'); ylabel('P_{out}, P_{sat} (dBm)');
yyaxis right;  hold on;
for i = 1:length(survey_stats.technology)
   ghs2(i) = yline(survey_stats.mean_Psat(i), 'Color', myColors(i,:), ...
   'LineWidth',2,'DisplayName',string(survey_stats.technology(i))); 
end
hold off; ax2 = gca; ax2.YColor = 'black';
ylim(yl2); % yticks([]); 
box on;
leg = legend('NumColumns',2,'Location','eastoutside', 'interpreter','none'); title(leg, 'Num. users            PA tech.');
end 
title({"Required P_{out} (at 6dB PBO) vs. mean P_{sat}","MBPAA, DMBA, HFC, EIRP "+EIRP_temp+"dBm"});

%% Figure D_3: SDMBA architecture: Required P_out (max) vs. mean P_sat for each technology

% Custom y-axis limits
yl1 = [0, 50];
for makefig = 1
figure;
for r = 1:length(K_range) - 1
subplot(2,2,r)
grid on; yyaxis left; hold on;
for i = 1:length(N_user_range)
   ghs1(i) = plot(Ntx_range, pout_1pa_sdmba(i,:,r),'k-o', 'LineStyle',linVec{i},...
   'Marker', markerVec{i}, 'MarkerFaceColor','none','DisplayName',num2str(N_user_range(i)));
end
hold off; ax1 = gca; ax1.YColor = 'black'; colororder('black'); 
ylim(yl1);
 xlabel('N_{TX}'); ylabel('P_{1dB_{PA}}, mean P_{sat} (dBm)');
yyaxis right;  hold on;
for i = 1:length(survey_stats.technology)
   ghs2(i) = yline(survey_stats.mean_Psat(i), 'Color', myColors(i,:), ...
   'LineWidth',2,'DisplayName',string(survey_stats.technology(i))); 
end
hold off; ax2 = gca; ax2.YColor = 'black';
ylim(yl1); yticks([]); box on;
if r == 4
    leg = legend('NumColumns',2,'Location','southoutside', 'interpreter','none'); title(leg, 'Num. users            PA tech.');
end
title("K = "+K_range(r));
end
end
sgtitle({"Required P_{1dB_{PA}} vs. mean P_{sat}","SDMBA, EIRP "+EIRP_temp+"dBm"});

%% Figure D_4: SDMBA architecture: Required P_out at 6dB PBO vs. mean P_sat for each technology

for makefig = 1
figure;
for r = 1:length(K_range) - 1
subplot(2,3,r)
grid on; yyaxis left; hold on;
for i = 1:length(N_user_range)
   ghs1(i) = plot(Ntx_range, pout_1pa_sdmba(i,:,r) - 6,'k-o', 'LineStyle',linVec{i},...
   'Marker', markerVec{i}, 'MarkerFaceColor','none','DisplayName',num2str(N_user_range(i)));
end
hold off; ax1 = gca; ax1.YColor = 'black'; colororder('black');
ylim(yl2);
 xlabel('N_{TX}'); ylabel('P_{out}, P_{sat} (dBm)');
yyaxis right;  hold on;
for i = 1:length(survey_stats.technology)
   ghs2(i) = yline(survey_stats.mean_Psat(i), 'Color', myColors(i,:), ...
   'LineWidth',2,'DisplayName',string(survey_stats.technology(i))); 
end
hold off; ax2 = gca; ax2.YColor = 'black';
ylim(yl2); yticks([]); box on;
if r == 4
    leg = legend('NumColumns',2,'Location','best', 'interpreter','none'); title(leg, 'Num. users            PA tech.');
end
title("K = "+K_range(r));
end
end
sgtitle({"PA: Required P_{out} (at 6dB PBO) vs. mean P_{sat}","SDMBA, EIRP "+EIRP_temp+"dBm"});

%% Figure D_5: HSA architecture: Required P_out (max) vs. mean P_sat for each technology

K_range_temp = [2,4,8,16,32];
for makefig = 1
figure;
for r = 1:length(K_range_temp)
   if r < 4
        subplot(2,3,r)
    else
        subplot(2,3, r+0.5)
   end
grid on; yyaxis left; hold on;
for i = 1:length(N_user_range)
   ghs1(i) = plot(Ntx_range, pout_1pa_hsa(i,:,r),'k-o', 'LineStyle',linVec{i},...
   'Marker', markerVec{i}, 'MarkerFaceColor','none','DisplayName',num2str(N_user_range(i)));
end
hold off; ax1 = gca; ax1.YColor = 'black'; colororder('black'); 
ylim(yl1);
 xlabel('N_{TX}'); ylabel('P_{1dB_{PA}}, mean P_{sat} (dBm)');
yyaxis right;  hold on;
for i = 1:length(survey_stats.technology)
   ghs2(i) = yline(survey_stats.mean_Psat(i), 'Color', myColors(i,:), ...
   'LineWidth',2,'DisplayName',string(survey_stats.technology(i))); 
end
hold off; ax2 = gca; ax2.YColor = 'black';
ylim(yl1); yticks([]); box on;
if r == 6
    leg = legend('NumColumns',2,'Location','best', 'interpreter','none'); title(leg, 'Num. users            PA tech.');
end
title("K = "+K_range(r));
end
end
sgtitle({"PA: Required P_{out} (max) vs. mean P_{sat}","HSA, EIRP "+EIRP_temp+"dBm"});

%% Figure: HSA architecture: Required P_out at 6dB PBO vs. mean P_sat for each technology

for makefig = 1
figure;
for r = 1:length(K_range)
subplot(2,3,r)
grid on; yyaxis left; hold on;
for i = 1:length(N_user_range)
   ghs1(i) = plot(Ntx_range, pout_1pa_hsa(i,:,r) - 6,'k-o', 'LineStyle',linVec{i},...
   'Marker', markerVec{i}, 'MarkerFaceColor','none','DisplayName',num2str(N_user_range(i)));
end
hold off; ax1 = gca; ax1.YColor = 'black'; colororder('black');
ylim(yl2);
 xlabel('N_{TX}'); ylabel('P_{out}, P_{sat} (dBm)');
yyaxis right;  hold on;
for i = 1:length(survey_stats.technology)
   ghs2(i) = yline(survey_stats.mean_Psat(i), 'Color', myColors(i,:), ...
   'LineWidth',2,'DisplayName',string(survey_stats.technology(i))); 
end
hold off; ax2 = gca; ax2.YColor = 'black';
ylim(yl2); yticks([]); 
box on;
if r == 5
    leg = legend('NumColumns',2,'Location','best', 'interpreter','none'); title(leg, 'Num. users            PA tech.');
end
title("K = "+K_range(r));
end
end
sgtitle({"PA: Required P_{out} (at 6dB PBO) vs. mean P_{sat}","HSA, EIRP "+EIRP_temp+"dBm"});
