function [pdc_dac, fs_dac, P_static, P_dynamic] = get_per_DAC_power(V_supply, NOB, BW, f_cor, OS_factor)
% Author: Natasha Birari, natashabirari@gmail.com, Aug 2022

% This function is used to estimate power consumption of a binary-weighted 
% current-steering (CS) DAC.
% It is a 1st order model

% INPUT:
% OS_factor : oversampling factor over nyquist rate i.e. over 2BW
% V_supply  : ANALOG supply voltage in V
% NOB       : DAC resolution in bits
% BW        : bandwidth in Hz
% f_cor     : DAC corner frequency, such that f_IF = f_cor + BW/2

% OUTPUT:
% pdc_dac   : per-DAC power consumption in W
% fs_dac    : DAC sampling frequency in Hz

% NOTE: Internal DAC parameters (f_cor, Io, Cp, beta) are taken from Cui (2005).
    
    % Io: unit current source based on LSB
    % It's value is bounded by noise floor (lower bound) and 
    % linearity req (upper bound)
    Io = 10e-6;   % 10 uA,
    
    % Cp: parasitic capacitance of each current switch  
    Cp = 1e-12;     % 1 pF
    
    % correcting factor for 2nd order effects
    beta = 1;                   % value=1 used in this model.
    
    ENOB = NOB-0.75;                              
    
    % max signal frequency
    fmax_sig = f_cor + BW/2;                            % Hz
    
    % DAC sampling frequency
    fs_dac = OS_factor*2*(fmax_sig);                    % Hz
    
    % static power consumption
    P_static = 0.5*V_supply*Io*(2.^ENOB - 1);           % W
    
    % dynamic power consumption
    P_dynamic = 0.5.*ENOB.*Cp.*(fs_dac).*V_supply.^2;   % W
    
    % total power consumption
    pdc_dac = beta*(P_static + P_dynamic);              % W
end