function [Pout_per_PA, feasible_idx] = get_per_PA_op(EIRP_user, Gtx_antenna, N_user, ntx_vec, arch)
% Author: Natasha Birari, natashabirari@gmail.com, Aug 2022

% This function calculates the required max output power per PA i.e. OP1db.
% To get average output power, subtract PBO value from function's output.
% Equations specified in main paper, Section IV-A: 'FEM Power Amplifier'

% NOTE: for sub-array architectures, the transmit power per antenna element or Ptx is 
% independent of 
% * the number of users (each subarray serves only 1 user) & 
% * the full array size Ntx (transmit power per element scales
%   with subarray size K only).
% Therefore the Ptx_per_antenna output is a scalar and is manually converted to
% a vector of size ntx_vec.


% INPUT:
% EIRP_user            : dBm, EIRP per user and using ... antenna elements
% Gtx_antenna          : dBi, gain per antenna element
% N_user               : scalar, or row vector for checking multiple values
% ntx_vec              : vector of feasible Ntx (mainly for subarrays)
% arch                 : "X" where X = 'MBPAA', '2DA', 'SDMBA', 'HFC' or 'HSA'
%                         If subarray, pass as row vector [arch, K]
%                         where K is number of antenna elements per subarray. 
%                       NOTE: K must be a scalar.

% OUTPUT: 
% Pout_all_user_per_PA : dBm, vector of size ntx_vec
% feasible_idx         : vector of size ntx_vec         

    % check for correct architecture name input
    if (arch ~= "MBPAA") & (arch ~= "DMBA") & (arch ~= "SDMBA") & (arch ~= "HFC") & (arch ~= "HSA") %#ok<*AND2>
        error('format of arch input is incorrect');
    end
    
    % if multiple values of N_user, convert to row vector
    if size(N_user,2) > 1
        N_user = N_user';
    end    
    
    % Define insertion loss and power back-off of circuit elements
    IL_PA = 1.15;       % dB, avg of various techs from Despoisse (2022)
    IL_BPF = 1;         % dB
    IL_WD  = 1;         % dB, only for SDMBA
    PBO = 6;            % dB, output backoff
    % PBO_in = 9.6;     % dB, input backoff
    min_PA_pout = 0;    % only cases where Pout_PA exceeds this threshold will be considered
    
    % get number of antenna elements per PA
    if arch(1) == "SDMBA"
        K = str2double(arch(2));            % subarray size
        num_antennas_per_PA = K;               
        IL_splitter_per_PA = IL_WD*(num_antennas_per_PA-1);      % 1dB splitter IL * num. splitters per PA
    elseif arch(1) == "HSA"
        K = str2double(arch(2));            % subarray size
        num_antennas_per_PA = 1;               
    else
        num_antennas_per_PA = 1;            % not a subarray
    end
    
    % compute transmit power per antenna required for link budget
    EIRP_array = EIRP_user + 10*log10(N_user);                  % dBm
    Ptx_per_array = EIRP_array - Gtx_antenna - 10*log10(ntx_vec);
    Ptx_per_antenna = Ptx_per_array - 10*log10(ntx_vec);        % dBm   
        
    % get output power per PA required for computed antenna Ptx
    Pout_per_PA = Ptx_per_antenna + 10*log10(num_antennas_per_PA)  + IL_PA + IL_BPF;  % dBm
    Pout_per_PA = Pout_per_PA + PBO;                          % dBm, Pout max
    
    % For SDMBA: add compensation for splitter insertion loss(es) and
    % additional BPFs
    if arch(1) == "SDMBA"
        Pout_per_PA = Pout_per_PA + IL_splitter_per_PA;  % dBm
    end
    
    % Get feasible values by removing any combination of Ntx and Pout:
    % where Pout less than min_Pout
    feasible_idx = double(Pout_per_PA >= min_PA_pout);    
    feasible_idx(feasible_idx  == 0) = NaN; % Avoid multiplication by zero               
    % where Number of antenna elements is less than number of users
    feasible_idx(ntx_vec < N_user) = NaN;       % for subarrays these values are already = 0 < N_user

%     % note: lines 68-71 do what line 64 does, but for when N_user is passed
%     % as a vector. It is for testing purposes, as in the main code we compute
%     % one value of N_user at a time
%     for U = 1:length(N_user)
%         if N_user(U) == 32 & (arch(1) == "MBPAA" | arch(1) == "2DA" | arch(1) == "HFC") %#ok<*OR2>
%             feasible_idx(U,1) = NaN;          % for case of Nuser = 32, Ntx = 16;
%         end
%     end

    % Required output power per PA for given users and ntx_vec
    % (feasible cases)
    Pout_per_PA = Pout_per_PA.*feasible_idx;
end
