function [Pdc_per_PA, feasible_array] = get_per_PA_pdc(pout_per_PA, feasible_array, tech, specs, category, pbo)
% Author: Natasha Birari, natashabirari@gmail.com, Aug 2022
%
% This function calculates the power consumption per PA for required output 
% power and the chosen technical specs. 
% 
% Required PA output power already takes architecture, number of users and 
% total number of antenna elements into account.
% 
% By default, required PA output power is specified as the maximum OP and
% not at power back-off (PBO). This can be toggled in the function's inputs
%
% The code also allows to check using the PA details from this work or from
% the benchmark model in Despoisse (2020).

% Last update to loaded specs: 01 Aug 2022

% INPUT:
% pout_per_PA       : dBm, output of function get_per_PA_op() 
% feasible_array    : vector of feasible Ntx values, size pout_per_PA
% specs             : source for PA specifications: choose between 'd' (Despoisse(2020))
%                     and 'm' (this model)
% tech              : PA technology "X" where X = 'CMOS_Bulk', 'CMOS_SOI',
%                     'SiGe', 'GaAs' or 'GaN'
% category          : specs category "X" where X = min, max or 0 (for mean).
% pbo               : toggle for reporting the P_dc at PBO
%                     If 0 - compute at max output power 
%                     else - compute at power backoff
%
% OUTPUT:
% Pdc_per_PA        : power consumption per PA in W, vector of size pout_per_PA
% feasible_array    : updated array of feasible Ntx values taking the tech's 
%                     saturation power (Psat) into account

    PBO = 6;        % dB, power back off   
    
    % Load technology specific specs
    if specs == 'm'   % this model
        specs_tab = readtable('supporting_files/PA_specifications_thiswork');
    elseif specs == 'd'   % Despoisse(2020)
        specs_tab = readtable('supporting_files/PA_specifications_benchmark');   % from component_analysis_PA.m
    end
      
    % Get index of specified tech 
    idx = find(strcmp(specs_tab.technology, tech));

    % Extract specs based on tech and category 
    if category == 'max'             % max values
        PA_tech_names =     specs_tab.technology;
        PAE_pbo =           specs_tab.max_PAE_avg(idx);
        psat =              specs_tab.max_Psat(idx);
    elseif category == 'min'         % min values
        PA_tech_names =     specs_tab.technology;
        PAE_pbo =           specs_tab.min_PAE_avg(idx);
        psat =              specs_tab.min_Psat(idx);
    else                    % averaged values
        PA_tech_names =     specs_tab.technology;
        PAE_pbo =           specs_tab.mean_PAE_avg(idx);
        psat =              specs_tab.mean_Psat(idx);
    end 
    
    % remove instances where required output power exceeds Psat + margin
    feasible_array(pout_per_PA > psat+1) = NaN;
    
    % Power consumption per PA, based on tech
    % (For reasonably high gain PAs, input power can be factored out)
    pdc_max = db2pow(pout_per_PA)./PAE_pbo;              % mW
    pdc_pbo = db2pow(pout_per_PA - PBO)./PAE_pbo;        % mW

    % Convert units and get feasible values
    pdc_max = pdc_max.*feasible_array.*1e-3;            % W
    pdc_pbo = pdc_pbo.*feasible_array.*1e-3;            % W
    
    % Report output value according to PBO toggle
    if pbo == 0 % not at PBO
        Pdc_per_PA = pdc_max;   
    else % at PBO
        Pdc_per_PA = pdc_pbo;                      
    end
end