function [total_power, total_power_breakdown, p_pa_tot] = get_total_power_hsa(num_components, power_components, ntx_vec, K_value)
% Author: Natasha Birari, natashabirari@gmail.com, Aug 2022
%
% This fuction multiplies the power consumption values of individual components
% and the number of components in the architecture to give the total power
% consumption of the architecture, as well as a breakdown of the total
% power based on the component.
%
% This fuction multiplies the power consumption values of individual components
% and the number of components in the architecture to give the total power
% consumption of the architecture, as well as a breakdown of the total
% power based on the component.

% It is customised for hybrid subarray (hsa) scenario.

% NOTE: all vectors in the cell array must be row vectors!
% NOTE: this code assumes that per-component P_dc will be a row vector of
% size 1x1 or 7x1 only. Any deviations from this will result in errors.
% VERY IMP:  Components order: DSP, DAC, mixer-LO, RFamp, PS, VGA, PA.
%       Indices in function are HARDCODED for the components' num and pow.
%
% ------------      List of dependent functions     ------------
% This function uses the following other functions:
% get_per_PA_op().m             PA required output power
% get_per_PA_pdc().m            PA power consumption
% get_per_DAC_power().m         DAC power consumption
% loss_budget_hsa().m          Signal splitting losses

% INPUTS:
% num_components: cell array of the number of components
% power_components: cell array of component powers OR parameters (DAC)
% ntx_vec: array of number of antenna elements, to pass on to PA power
%           function. NOT Ntx_range, but rather the chosen row of ntx_mat.
% K_value: number of elements per sub-array i.e. number of elements
%          connected to one PA. Passed on to PA power function for HSA case.
%
% OUTPUTS: 
% total_power:      Total system power consumption, in mW
% total_power_breakdown: Component-wise breakdown of system power consumption, in mW
% p_pa_tot:         Only PA power consumption, in mW
                 

disp('Computing power for HSA')

[s,d] = cellfun(@size, num_components);
lmax = max([s,d]);

% DSP ------------------------------------------

nr = num_components{1};
pw = power_components{1};
p_bbu_tot = nr.*pw;

if max(size(p_bbu_tot)) == 1
    p_bbu_br = p_bbu_tot.*ones(1,lmax);    
elseif max(size(p_bbu_tot)) == lmax
    p_bbu_br = p_bbu_tot;
else
    error('incorrect dimensions of vector p_bbu_tot');
end

% DAC ------------------------------------------

nr = num_components{2};
props = power_components{2};
[Vdd, enob, BW, f_cor, OS] = deal(props(1),props(2),props(3),props(4),props(5));
pw = get_per_DAC_power(Vdd, enob, BW, f_cor, OS);   % W
pw = round(pw*1000);                                % mW
p_dac_tot = nr.*pw;

if max(size(p_dac_tot)) == 1
    p_dac_br = p_dac_tot.*ones(1,lmax);    
elseif max(size(p_dac_tot)) == lmax
    p_dac_br = p_dac_tot;
else
    error('incorrect dimensions of vector p_dac_tot');
end

% mixer LO -------------------------------------

nr = num_components{3};
pw = power_components{3};
p_mxLO_tot = nr.*pw;

if max(size(p_mxLO_tot)) == 1
    p_mxLO_br = p_mxLO_tot.*ones(1,lmax);    
elseif max(size(p_mxLO_tot)) == lmax
    p_mxLO_br = p_mxLO_tot;
else
    error('incorrect dimensions of vector p_mxLO_tot');
end

% PS -------------------------------------------

nr = num_components{5};
pw = power_components{5};
p_ps_tot = nr.*pw;

if max(size(p_ps_tot)) == 1
    p_ps_br = p_ps_tot.*ones(1,lmax);    
elseif max(size(p_ps_tot)) == lmax
    p_ps_br = p_ps_tot;
else
    error('incorrect dimensions of vector p_ps_tot');
end

% VGA -------------------------------------------

nr = num_components{6};
pw = power_components{6};
p_vga_tot = nr.*pw;

if max(size(p_vga_tot)) == 1
    p_vga_br = p_vga_tot.*ones(1,lmax);    
elseif max(size(p_vga_tot)) == lmax
    p_vga_br = p_vga_tot;
else
    error('incorrect dimensions of vector p_vga_tot');
end

% PA -------------------------------------------

nr = num_components{7};
props = power_components{7};

[eirp, gtx_antenna, n_user, PAtech, specs, cat] = deal(str2num(props(1)), ...
    str2num(props(2)), str2num(props(3)), props(4), ...
    props(5),props(6));

[pout_per_pa, feasible_array] = get_per_PA_op(eirp, gtx_antenna, n_user, ntx_vec, ["HSA", K_value]); % dBm
[p_per_pa,feasible_array] = get_per_PA_pdc(pout_per_pa, feasible_array, PAtech, specs, cat, 0); % W

p_pa_tot = p_per_pa.*nr.*1e3;   % W -> mW

if max(size(p_pa_tot)) == 1
    p_pa_br = p_pa_tot.*ones(1,lmax);    
elseif max(size(p_pa_tot)) == lmax
    p_pa_br = p_pa_tot;
else
    error('incorrect dimensions of vector p_pa_tot');
end

% RF Amps  ----------------------------------------

for rfcode = 1

BB_signal_output_power = -2;              % dBm
PA_input_min_power = 0;                   % dBm
PBO = 6;                                  % dB

% This code computes Pout at it's max value.
% To get value at PBO instead - toggle line below, MANUAL
% pout_pa = pout_pa - PBO;              % dBm                          

[mat_gsplitteramp, ~,...
 mat_gpd, mat_npd, mat_gpa, ...
 mat_tot_nsplitteramp, ...
 mat_tot_npd, ~, ~] = loss_budget_hsa(BB_signal_output_power, ...
                                      PA_input_min_power, ...
                                      pout_per_pa, feasible_array, ...
                                      ntx_vec, K_value, [n_user]);                                  
                                  
                                  
% Splitter Amp
nr = mat_tot_nsplitteramp;
g = mat_gsplitteramp;
pw_range = power_components{4};        

for i = 1:length(nr)
    if g(i) < 10
        pw(i) = pw_range(1);
    elseif g(i) >= 10 && g(i) < 15
        pw(i) = pw_range(2);
    else
        pw(i) = pw_range(3);
    end
end

p_rfamp_split_tot = nr.*pw;                         % mW

if max(size(p_rfamp_split_tot)) == 1
    p_rfamp_split_br = p_rfamp_split_tot.*ones(1,lmax);    
elseif max(size(p_rfamp_split_tot)) == lmax
    p_rfamp_split_br = p_rfamp_split_tot;
else
    error('incorrect dimensions of vector p_rfamp_split_tot');
end

% Combiner Amp
p_rfamp_combine_tot = 0;                            % mW
p_rfamp_combine_br = p_rfamp_combine_tot.*ones(1,lmax);

% Pre-driver (PD)
nr = mat_tot_npd;
g = mat_gpd;
pw_range = power_components{4};        

for i = 1:length(nr)
    if g(i) < 10
        pw(i) = pw_range(1);
    elseif g(i) >= 10 && g(i) < 15
        pw(i) = pw_range(2);
    else
        pw(i) = pw_range(3);
    end
end

p_rfamp_pd_tot = nr.*pw;                    % mW

if max(size(p_rfamp_pd_tot)) == 1
    p_rfamp_pd_br = p_rfamp_pd_tot.*ones(1,lmax);    
elseif max(size(p_rfamp_pd_tot)) == lmax
    p_rfamp_pd_br = p_rfamp_pd_tot;
else
    error('incorrect dimensions of vector p_rfamp_pd_tot');
end

% All RF Amps
p_rfamp_tot = p_rfamp_split_tot + p_rfamp_combine_tot + p_rfamp_pd_tot; % mW
p_rfamp_br = p_rfamp_split_br + p_rfamp_combine_br + p_rfamp_pd_br;     % mW

end

% Total ----------------------------------------

total_power_raw = (p_bbu_tot + p_dac_tot + p_mxLO_tot + p_rfamp_tot + p_ps_tot + p_vga_tot + p_pa_tot); % mW
total_power = total_power_raw.*feasible_array;  % mW

total_power_breakdown_raw = [p_bbu_br; p_dac_br; p_mxLO_br; p_rfamp_br; p_ps_br; p_vga_br; p_pa_br];
total_power_breakdown = total_power_breakdown_raw.*feasible_array;

end