function[mat_gpd, mat_npd, mat_gpa, mat_tot_npd, mat_ippa] ...
                = loss_budget_dmba(input_power, input_PA_min, pout_per_pa, ...
                                    feasible_idx, Ntx_range, N_user_range)
% Author: Natasha Birari, natashabirari@gmail.com, Aug 2022
%
% Loss modelling for DMBA
% This function creates a signal distribution structure using given inputs
% and returns the number + per-element gain/loss of the following elements:
% splitters, splitter amplifiers, combiners, combiner amplifiers,
% pre-drivers.
%
% This function is customised for the fully-digital (dmba) scenario.
%
% Signal path:
% BB OUTPUT --> DAC --> BPF --> mxLO --> PD amp (if needed)--> PA
%
% INPUT: 
% input_power:      input power to RFChain from Baseband, dBm
% input_PA_min:     input power to main PA from predriver, dBm. Default = 0;
% pout_per_pa:      row vector
% feasible_idx:     feasible values of array sizes
% Ntx_range:        antenna array sizes, row vector
% N_user_range:     number of users range, row vector

% Default values: 
% Gain dB, IL dB
DAC = [0 -0.5];
BPF = [0 -1];
mxLO = [0 -0.5];

PDAmp = [5 -1];         % predriver, variable gain
PA = [18 -1.15];        % main PA, variable gain

% number of components per channel, indep. of Ntx or N_user
num_DAC = 1;
num_BPF = 1;
num_mxLO = 1;
num_PA = 1;

%% Setup

% Check input signal strength. Reason: mxLO output should be between -10 and 0 dBm
if input_power < -10 || input_power > 0 
    error('Input power to RF chain must be between -10 and 0 dBm');
end

% Predefine empty matrices - per channel i.e. one data stream to one
% antenna element

mat_gpd = feasible_idx;     % predriver amps gain
mat_npd = feasible_idx;     % predriver amps number
mat_ippa = feasible_idx;    % PA input power
mat_gpa = feasible_idx;     % PA gain

% Predefine empty matrices - totals i.e. for given value of Ntx and Nuser
mat_tot_npd = feasible_idx; % total number of predriver amps


%% Loop over all users and Ntx values
for i = 1:length(N_user_range)
    for j = 1: length(Ntx_range)
        
        if ~isnan(feasible_idx(i,j))
            
            U = N_user_range(i);
            N = Ntx_range(j);

            num_PDAmp = 1;

            %% Equations between BB and PD

            op  = input_power + sum(DAC).*num_DAC;
            op  = op + sum(BPF).*num_BPF;
            op2  = op + sum(mxLO).*num_mxLO;
            
            ip_PD = op2;

            %% PD
            % if output power of previous stage < IL of PD Amp,
            % compensate PD Amp IL and bring to input_PA_min (default: 0dBm)
            % else, no PD needed
            PDAmpT = PDAmp;
            if ip_PD <= PDAmpT(2)               
                PDAmpT(1) = -ip_PD - PDAmpT(2) + input_PA_min;
                num_PDAmp = ceil(PDAmpT(1)/15);      % multiples of 15 dB amps
                op_PD = ip_PD + sum(PDAmpT).*1;
            else
                PDAmpT(1) = 0;
                num_PDAmp = 0;
                op_PD = ip_PD + sum(PDAmpT).*num_PDAmp;
            end
            
            ip_PA = op_PD;

            %% PA 
            % with pout_per_pa already defined, this section calculates the
            % PA gain required to get the required pout.
            PAT = PA;
            PAT(1) = pout_per_pa(i,j) - PAT(2) - ip_PA;
            op_PA = ip_PA + sum(PAT).*num_PA;
            
            %% totals
            
            num_tot_PDAmp = N.*num_PDAmp;

            %% save data

            mat_gpd(i,j)          = PDAmpT(1);
            mat_npd(i,j)          = num_PDAmp;
            
            mat_ippa(i,j)         = ip_PA;
            mat_gpa(i,j)          = PAT(1);
            
            mat_tot_npd(i,j)      = num_tot_PDAmp;
        
        else
            continue % NaN values
        end
    end
end

% note: adjust RowNames as necessary

% view any mat as table, e.g. for total num PD amps
% tab = array2table(mat_tot_npd, 'RowNames',cellstr(num2str(N_user_range'))','VariableNames',{'16','32','64','128','256','384','512'});

end