function[mat_gsplitteramp, mat_nsplitteramp, mat_gcombineramp, mat_ncombineramp,...
    mat_gpd, mat_npd, mat_gpa, mat_tot_nsplitteramp, mat_tot_ncombineramp,...
    mat_tot_npd, mat_opsplit, mat_opcomb, mat_ippa]  = loss_budget_fc_analog(input_power, input_PA_min, ...
                        pout_per_pa, feasible_idx, Ntx_range, N_user_range)
% Author: Natasha Birari, natashabirari@gmail.com, Aug 2022
%
% Loss modelling for MBPAA/HFC
% This function creates a signal distribution structure using given inputs
% and returns the number + per-element gain/loss of the following elements:
% splitters, splitter amplifiers, combiners, combiner amplifiers,
% pre-drivers.
% 
% This function is customised for the fully-connected (fc) hybrid/analog
% (ah) scenario.
% 
% Signal path:
% BB --> DAC --> mxLO --> splitter network --> PS and VGA --> combiner
% network --> PD amp (if needed) --> PA
%
% INPUT: 
% input_power:      input power to RFChain from Baseband, dBm
% input_PA_min:     input power to main PA from predriver, dBm. Default = 0;
% pout_per_pa:      row vector
% feasible_idx:     feasible values of array sizes
% Ntx_range:        antenna array sizes, row vector
% N_user_range:     number of users range, row vector

% Default values: 
% Gain dB, IL dB
DAC         = [0 -0.5];
BPF         = [0 -1];          
mxLO        = [0 -0.5];

Splitter    = [-3 -1];     % 3 dB splitting loss
Combiner    = [-3 -1];     % 1 dB combining loss

SplitterAmp = [12 -1];  
CombinerAmp = [4 -1];

PS          = [2 0];        % active VM phase shifter
VGA         = [12 -1];       % pre-combining amplifier, variable gain

PDAmp       = [5 -1];       % predriver, variable gain
PA          = [18 -1.15];   % main PA, variable gain

% number of components per channel, indep. of Ntx or N_user
num_DAC     = 1;
num_BPF     = 1;
num_mxLO    = 1;
num_PS      = 1;
num_VGA     = num_PS;
num_PA      = 1;

%% Setup

% Check input signal strength, as mxLO output should be between -10 and 0 dBm
if input_power < -10 || input_power > 0 
    error('Input power to RF chain must be between -10 and 0 dBm');
end

% Predefine empty matrices - per channel i.e. one data stream to one
% antenna element

mat_gsplitteramp = feasible_idx;        % splitter amps gain
mat_nsplitteramp = feasible_idx;        % splitter amps number
mat_opsplit = feasible_idx;             % splitter amps output power
mat_gcombineramp = feasible_idx;        % combiner amps gain
mat_ncombineramp = feasible_idx;        % combiner amps number
mat_opcomb = feasible_idx;              % combiner amps output power
mat_gpd = feasible_idx;                 % predriver amps gain
mat_npd = feasible_idx;                 % predriver amps number
mat_ippa = feasible_idx;                % PA input power
mat_gpa = feasible_idx;                 % PA gain

% Predefine empty matrices - totals i.e. for given value of Ntx and Nuser

mat_tot_nsplitteramp = feasible_idx;    % total number of splitter amps
mat_tot_ncombineramp = feasible_idx;    % total number of combiner amps
mat_tot_npd = feasible_idx;             % total number of predriver amps


%% Loop over all users and Ntx values
for i = 1:length(N_user_range)
    for j = 1: length(Ntx_range)
        
        if ~isnan(feasible_idx(i,j))
            
            U = N_user_range(i);
            N = Ntx_range(j);

            % Splitting
            num_Splitter = floor(log2(N));
            if N == 512     % special case: 1 amp per 2 splitters, with no pre-splitter amp
                num_SplitterAmp = floor(num_Splitter/2);   
            else            % default case: 1 amp per 3 splitters, starting with pre-splitter
                num_SplitterAmp = floor(num_Splitter/3)+1;  
            end
            SplitterAmpT = SplitterAmp; % adjust SplitterAmp Gain for large arrays
            if N >= 256 && N < 512
                SplitterAmpT(1) = SplitterAmpT(1)+1;
            end
            
            % PS & VGA Gain
            if U == 2
                VGA(1) = 4;
            elseif U == 4
                VGA(1) = 8;
            else
                VGA(1) = 9;
            end
            
            % Combiner Amps
            num_Combiner = floor(log2(U));
            if U == 16
                num_CombinerAmp = 2;
            elseif U == 32
                num_CombinerAmp = 4;
            else
                num_CombinerAmp = 0;
            end
            CombinerAmpT = CombinerAmp;
            % pre-driver
            num_PDAmp = 1;

            %% Equations between Baseband (BB) and pre-driver (PD)

            op  = input_power +  sum(DAC).*num_DAC;
            op  = op + sum(BPF).*num_BPF;
            op  = op + sum(mxLO).*num_mxLO;
            op  = op + sum(Splitter).*num_Splitter;
            op1 = op + sum(SplitterAmpT).*num_SplitterAmp;
            op  = op1 + sum(PS).*num_PS;
            op  = op + sum(VGA).*num_VGA;
            op  = op + sum(Combiner).*num_Combiner;
            op2 = op + sum(CombinerAmpT).*num_CombinerAmp;

            ip_PD = op2;

            %% PD
            % if output power OP < IL of PD Amp,
            % compensate PD Amp IL and bring to input_PA_min (default: 0dBm)
            % else, no PD needed
            PDAmpT = PDAmp;
            if ip_PD <= PDAmpT(2)               
                PDAmpT(1) = -ip_PD - PDAmpT(2) + input_PA_min;
                num_PDAmp = ceil(PDAmpT(1)/15);      % multiples of 15 dB amps
                op_PD = ip_PD + sum(PDAmpT).*1;
            else
                PDAmpT(1) = 0;
                num_PDAmp = 0;
                op_PD = ip_PD + sum(PDAmpT).*num_PDAmp;
            end
            
            ip_PA = op_PD;

            %% PA 
            % with pout_per_pa already defined, this section calculates the
            % PA gain required to get the required pout.
            PAT = PA;
            PAT(1) = pout_per_pa(i,j) - PAT(2) - ip_PA;
            op_PA = ip_PA + sum(PAT).*num_PA;
            
            %% totals
            
            if N == 512     % special case: 1 amp per 2 splitters
                num_tot_SplitterAmp = U.*(sum(2.^(2.*(1:1:floor(log2(N)/2)))));   
            else            % default case: 1 amp per 3 splitters
                num_tot_SplitterAmp = U.*(sum(2.^(3*(0:1:floor(log2(N)/3)))));
            end
                        
            num_tot_CombinerAmp = N.*(num_CombinerAmp);
            num_tot_PDAmp = N.*num_PDAmp;

            %% save data

            mat_gsplitteramp(i,j) = SplitterAmpT(1);
            mat_nsplitteramp(i,j) = num_SplitterAmp;
            mat_opsplit(i,j)      = op1;
            
            mat_gcombineramp(i,j) = CombinerAmpT(1);
            mat_ncombineramp(i,j) = num_CombinerAmp;
            mat_opcomb(i,j)       = op2;
            
            mat_gpd(i,j)          = PDAmpT(1);
            mat_npd(i,j)          = num_PDAmp;
            
            mat_ippa(i,j)         = ip_PA;
            mat_gpa(i,j)          = PAT(1);
            
            mat_tot_nsplitteramp(i,j) = num_tot_SplitterAmp;
            mat_tot_ncombineramp(i,j) = num_tot_CombinerAmp;
            mat_tot_npd(i,j) = num_tot_PDAmp;
                    
        else
            continue % NaN values
        end
    end
end

% note: adjust RowNames as necessary

% view any mat as table, e.g. for total num PD amps by:
% tab = array2table(mat_tot_npd, 'RowNames',cellstr(num2str(N_user_range'))','VariableNames',{'16','32','64','128','256','384','512'});

end