function[mat_gsplitteramp, mat_nsplitteramp,...
    mat_gpd, mat_npd, mat_gpa, mat_tot_nsplitteramp, ...
    mat_tot_npd, mat_opsplit, mat_ippa]  = loss_budget_hsa(input_power, input_PA_min, ...
                        pout_per_pa, feasible_idx, Ntx_range, K_range, N_user_range)
% Author: Natasha Birari, natashabirari@gmail.com, Aug 2022
%
% Loss modelling for HSA
% This function creates a signal distribution structure using given inputs
% and returns the number + per-element gain/loss of the following elements:
% splitters, splitter amplifiers, combiners, combiner amplifiers,
% pre-drivers
%
% This function is customised for the hybrid subarray (hsa) scenario.
%
% Signal path:
% BB output --> DAC --> BPF --> mxLO --> splitter network --> PS --> 
% PD amp (if needed)--> PA

% INPUT: 
% input_power:      input power to RFChain from Baseband, dBm
% input_PA_min:     input power to main PA from predriver, dBm. Default = 0;
% pout_per_pa:      row vector or matrix, same dimensions as Ntx_range
% feasible_idx:     feasible values of array sizes
% Ntx_range:        row vector or matrix, with num. rows = num_config
% K_range:          scalar (Ntx_range = vector) or row vector (Ntx_range = matrix)
% N_user_range:     SCALAR, because Ntx_range is provided as a row vector/matrix for
%                   fixed N_user.
%
% OUTPUT: a matrix (1D/2D) for Ntx antenna elements along columns and
% num_config values of K along rows. If K is passed as a vector, output is
% 2D, else 1D.
% Note: pout_per_pa input already takes the K-dependent splitting loss into
% account.

% Default values: 
% Gain dB, IL dB
DAC         = [0 -0.5];
BPF         = [0 -1];          
mxLO        = [0 -0.5];

Splitter    = [-3 -1];     % 3 dB splitting loss
SplitterAmp = [8 -1];      % 1 dB combining loss

PS          = [2 0];       % active VM phase shifter
    
PDAmp       = [5 -1];         % predriver, variable gain
PA          = [18 -1.15];     % main PA, variable gain

% number of components per channel, indep. of Ntx or N_user
num_DAC     = 1;
num_BPF     = 1;
num_mxLO    = 1;
num_PS      = 1;
num_PA      = 1;

%% Setup

% Predefine empty matrices - per channel i.e. one data stream to one
% antenna element

mat_gsplitteramp = feasible_idx;        % splitter amps gain
mat_nsplitteramp = feasible_idx;        % splitter amps number
mat_opsplit = feasible_idx;             % splitter amps output power

mat_gpd = feasible_idx;                 % predriver amps gain
mat_npd = feasible_idx;                 % predriver amps number
mat_ippa = feasible_idx;                % PA input power
mat_gpa = feasible_idx;                 % PA gain

% Predefine empty matrices - totals i.e. for given value of Ntx and Nuser

mat_tot_nsplitteramp = feasible_idx;    % total number of splitter amps
mat_tot_npd = feasible_idx;             % total number of predriver amps


%% Loop over all user, K and Ntx values
for h = 1:length(K_range)
for i = 1:length(N_user_range)
    for j = 1: length(Ntx_range)
        
        if ~isnan(feasible_idx(h,j))
            
            K = K_range(h);            
            U = N_user_range(i);
            N = Ntx_range(h,j);
            M = N/K;                            % Num. RFCs

            num_Splitter = floor(log2(K));
            num_SplitterAmp = floor(num_Splitter/2)+1;   % default case: 1 amp per 2 splitters, starting with pre-splitter
            num_PDAmp = 1;

            % adjust SplitterAmp Gain for large arrays
            % currently 0 for testing
            SplitterAmpT = SplitterAmp;
            if N >= 256 && N < 512
                SplitterAmpT(1) = SplitterAmpT(1)+0;
            end

            %% Equations between Baseband (BB) and pre-driver (PD)

            op  = input_power + sum(DAC).*num_DAC;
            op  = op + sum(BPF).*num_BPF;
            op  = op + sum(mxLO).*num_mxLO;
            op  = op + sum(Splitter).*num_Splitter;
            op1 = op + sum(SplitterAmpT).*num_SplitterAmp;
            op2  = op1 + sum(PS).*num_PS;
            
            ip_PD = op2;

            %% PD
            % if output power of previous stage < IL of PD Amp,
            % compensate PD Amp IL and bring to input_PA_min (default: 0dBm)
            % else, no PD needed
            PDAmpT = PDAmp;
            if ip_PD <= PDAmpT(2)               
                PDAmpT(1) = -ip_PD - PDAmpT(2) + input_PA_min;
                num_PDAmp = ceil(PDAmpT(1)/15);      % multiples of 15 dB amps
                op_PD = ip_PD + sum(PDAmpT).*1;
            else
                PDAmpT(1) = 0;
                num_PDAmp = 0;
                op_PD = ip_PD + sum(PDAmpT).*num_PDAmp;
            end
            
            ip_PA = op_PD;

            %% PA 
            % with pout_per_pa already defined, this section calculates the
            % PA gain required to get the required pout.
            PAT = PA;
            PAT(1) = pout_per_pa(h,j) - PAT(2) - ip_PA;
            op_PA = ip_PA + sum(PAT).*num_PA;
            
            %% totals
            
            % default case: 1 amp per 2 splitters, starting with pre-splitter; 
            % num. RFC x SplitterAmps per PS group.
            num_tot_SplitterAmp = M.*(sum(2.^(2.*(0:1:floor(log2(K)/2)))));   
            num_tot_PDAmp = N.*num_PDAmp;

            %% save data

            mat_gsplitteramp(h,j) = SplitterAmpT(1);
            mat_nsplitteramp(h,j) = num_SplitterAmp;
            mat_opsplit(h,j)      = op1;
            
            mat_gpd(h,j)          = PDAmpT(1);
            mat_npd(h,j)          = num_PDAmp;
            
            mat_ippa(h,j)         = ip_PA;
            mat_gpa(h,j)          = PAT(1);
            
            mat_tot_nsplitteramp(h,j) = num_tot_SplitterAmp;
            mat_tot_npd(h,j)          = num_tot_PDAmp;
        
        else
            continue % NaN values
        end
    end
end
end

% note: adjust RowNames as necessary

% view any mat as table, e.g. for total num PD amps
% tab = array2table(mat_tot_npd, 'RowNames',cellstr(num2str(N_user_range'))','VariableNames',{'16','32','64','128','256','384','512'});

end