%% ----------- INTRODUCTION -----------  %%
% Author: Natasha Birari, natashabirari@gmail.com, Aug 2022
%
% This script is the main model definition for fully-connected architectures
% as defined in this work i.e. MBPAA, HFC and DMBA architectures.
% It uses the provided user inputs with the coded model parameters to
% calculate the system- and component-level power consumption and generate
% plots w.r.t. the array size (Ntx)
%
% This script works together with other functions for specific calculations
% (see list below)
%
% System-level USER inputs are defined in the beginning of the script, followed
% by topology and component-level inputs and modelling. See the main paper
% for descriptions, examples and limitations of the user inputs.

% ---------     Dependent functions     ---------
% This script uses the following functions:
% get_total_power_fc_ah().m     For fully-connected analog
% get_total_power_dmba().m      For fully-connected digital
% plotSingleBarsPdc().m         Plotting

% case_num: use case as defined in the main paper.
% N_user:   number of simultaneous users as defined in the main paper
%           Not all combinations of case_num and N_user are feasible

%% Base settings

% clear; clc;            
rng(1);                 % random seed
fontsize = 11;
set(0, 'DefaultAxesTickLabelInterpreter', 'none'); 
% linecolors = ['k','g','b','m','c','r']; % upto 6 configurations
linecolors2 = ['r';'b';'k']; % upto 6 configurations
linecolors_rgb = [0 0 0; 0 1 0; 0 0 1; 1 0 1; 0 1 1; 1 0 0]; % upto 6 configurations

%% ---------------------------- USER INPUTS -------------------------------
% Set these values to choose which scenario to calculate/plot       

% choose case 1, 2 or 3
case_num = 3;   

% Set number of simultaneous users 
N_user_range = [2, 4, 8, 16, 32];  % Possible range of values
N_user = N_user_range(3);          % Chosen value (scalar)       

% If line of sight scenario, set 0
% If non line of sight scenario, set 1 and define LOS_ratio > 0
isNLOS = 0;       
LOS_ratio_default = 1;

% EIRP toggle
% '0' for default 40 dBm EIRP, '1' for calculated average EIRP which
% considers LOS probability
% Can manually set different default EIRP than 40dBm
EIRP_default = 40;      % dBm
EIRP_toggle = 0;

% PA technology to analyse
% NOTE: insert within " ". Order: 'CMOS_Bulk', 'CMOS_SOI', 'SiGe', 'GaAs', 'GaN'
PAtech_name = "GaN";     

%% ------------------------ MAIN_RESULT_OPTIMA --------------------------
% If script is being run from main_result_optima.m :
% Overwrite selected user inputs with the value in main_result_optima.m

% check if main_result_optima script is being run
ise = evalin( 'base', 'exist(''create_data'',''var'') == 1' ); 
% if main_result_optima is active, check if new P_dc data should be created
if ise 
    if create_data == 'y' 
        % Overwrite inputs
        case_num = case_num_run;
        N_user_range = N_user_range_run;
        N_user = N_user_run;
        isNLOS = isNLOS_run;
        LOS_ratio_default = LOS_ratio_default_run;
        PAtech_name = PAtech_run;
        EIRP_toggle = EIRP_toggle_run;
        EIRP_default = EIRP_default_run;
    end
end

%% ---------------------------- CHECK  -------------------------------
% check for case-user compatibility
if (case_num > 2 && N_user >= 16) || (case_num > 1 && N_user >= 32)
    fprintf("Case number: "+case_num+", N_user: "+N_user+"\n");
    error('This combination of "case_num" and "N_user" is incompatible, check docs.');
else
    fprintf("Case number: "+case_num+", N_user: "+N_user+"\n");
end

%% Constants

c = 3e8;                            % m/s, speed of light
kB = physconst("Boltzman");         % Boltzman constant
f_OP = 28e9;                        % Hz, centre frequency
distance = 0.1;                     % km, distance between BS & users

%% Signal Properties

bandwidths = [100 200 400]*1e6;                 % Hz
datarates_physical = [300 780 1090]*1e6;        % bps, datarate per user of PHY link based on MCS and Shannon
datarates = datarates_physical;                 % bps, actual expected throughput per user based on reality
SNR_req = [8.5 11.45 7.51];                     % dB
mod_order = [16 64 64];                         % modulation, M-QAM
C_R = [0.754 0.650 0.455];                      % coding rate based on M

cases = [bandwidths; datarates; SNR_req; mod_order; C_R];   % store in mat.

% NOTE: to use a different bandwidth, edit the following vars:
% * bandwidths array - add the desired bandwidth
% * mod_order array - add the chosen modulation order
% * C_R array - add the chosen coding rate based on mod. order
% * datarates_physical array - look up based on M, C_R and SNR and add 
% * SNR_req array - calculate based on MCS and add

%% Link budget per user
% Model details: UMi Open Square scenario, LOS and NLOS, CI model from
% Haneda (2016), 3GPP mmW channel with BW at least 100MHz
% Multiuser aspect is considered directly in the PA output power calculation
% (see func get_per_PA_op()).

% Set LOS_ratio = percentage of users with LOS link
if isNLOS == 0                        % no NLOS scenario
    LOS_ratio = 1;
else
    if N_user >= 0             
        LOS_ratio = LOS_ratio_default;  % Based on Haneda (2016)              
    else                              
        LOS_ratio = 1;            % no NLOS scenario
    end
end

% Base station
% NOTE: Total array gain Gtx is calculated in get_per_PA_op(), where
%       Gtx = Gtx_antenna + 10*log10(number of antennas)
Gtx_antenna = 3;                    % dBi, gain per antenna element

% Channel parameters
PL_coeff = [1.85 2.89]';        % path loss coefficient, LOS and NLOS
SF = [4.2 7.1]';                % dB, shadow fading, LOS and NLOS

% path loss (distance in m), vector [LOS and NLOS]
PL = round(20*log10(4*pi*f_OP/c) + PL_coeff*10*log10(distance*1e3) + SF,0);    % dB

% bandwidth dependent channel noise, Despoisse (2020)
N_AWGN = round(pow2db(kB*298.*bandwidths),1);  % dB

% User Equipment
Nr = 4;                                     % Despoisse (2020), 2x2 panel, 1 panel per UE
Grx_antenna = 5;                            % dBi, gain per antenna element, 3GPP RAN 1 link budget discussions: 0dbi for FR2, 5dbi for FR2
Grx = Grx_antenna + round(10*log10(Nr));    % dB, receiver antenna gain
NF = 4;                                     % dB, receiver noise figure, Despoisse (2020)[27]

% Link budget relations
Prx = SNR_req + NF;                                 % dB, receive power at UE
EIRP_array = Prx + N_AWGN + PL - Grx;               % dB, transmit EIRP
EIRP_array = EIRP_array + 30;                       % dB -> dBm, transmit EIRP
% Ptx = EIRP_array - Gtx;                           % dBm, array transmit
% dimensions of EIRP_array: [LOS; NLOS] x number of input bandwidths

% EIRP calculation, LOS and NLOS
EIRP_los = db2pow(EIRP_array(1,:)).*LOS_ratio;      % LOS paths probability
EIRP_nlos = db2pow(EIRP_array(2,:)).*(1-LOS_ratio); % NLOS paths probability
EIRP_avg = pow2db(EIRP_los + EIRP_nlos);            % average EIRP

% Choose default or calculated EIRP for model
if EIRP_toggle == 1
    EIRP = EIRP_avg(case_num); 
elseif EIRP_toggle == 0
    EIRP = EIRP_default;
else
    EIRP = EIRP_default;
    disp('Unknown input specified for EIRP_toggle. Now using default value')
end

%% Topology sizing

%% Step 0: configure common params

% Number of transmit antenna elements
Ntx_range = [16, 32, 64, 128, 256, 512];

% Components order: BBU, DAC, mixer-LO, RFamp, PS, VGA, PA
comp_names = {{'BBU','DAC','mixer-LO','LCamp','PS', 'VGA','PA'}};

% PA tech names
PA_tech_names = {'CMOS_Bulk', 'CMOS_SOI', 'SiGe', 'GaAs', 'GaN'};

% NOTE:
% To remove components or PA techs, set their component-specific
% power consumption value to 0 mW further down the script.
% To add new components or add new PA techs, change the following scripts:
% get_total_power()
% loss_budget()

%% Step 1: basic setup of architecture - RF chain, PS, PA
% The model is built on the assumption that 
% Number of RF chains = number of users.

% M_x:              Number of RF chains for arch x
% num_PA_x:         Total number of PAs for arch x
% num_PS_x:         Total number of phase shifters/VGAs for arch x
% num_BBU_ops_x:    Total number of digital beamforming operations 
%                   per user, for arch x
% 
% In this section, the number of RF amplifiers is passed as 1 but 
% the real value is calculated in func get_total_power_x()
% based on the signal branching model in loss_budget_x()

% Fully connected architectures 
% These are also identifiers in other scripts, & should be changed with care
arch_names_fc = {'MBPAA','DMBA','HFC'};

% Set up MBPAA ------------------------------------------------------------

M_mbp = N_user;               % (Min required number of users)
num_PA_mbp = Ntx_range;
num_PSVGA_mbp = M_mbp.*Ntx_range;
num_BBU_ops_mbp = 0; 

% Set up DMBA (digital array) ----------------------------------------------

M_dmba = Ntx_range;              % Number of RF chains for DMBA
num_PA_dmba = M_dmba;             % Total number of PAs
num_BBU_ops_dmba = N_user.*Ntx_range;

% Set up HFC --------------------------------------------------------------

M_hfc = N_user;                  % (Min required number of users)
num_PA_hfc = Ntx_range;
num_PSVGA_hfc = M_hfc.*Ntx_range;
hybrid_factor_BBU = 0.8;         % difference of BBU intensity b/w DMBA and hybrid
num_BBU_ops_hfc = N_user.*Ntx_range.*hybrid_factor_BBU;

%% Step 2: Set number of components 
% Order according to 'comp_names'
% Number of components:     'BBU'           'DAC'       'mixer-LO'    'RFamp'      'PS'           'VGA'               'PA'

num_comps_mbp =         {num_BBU_ops_mbp     M_mbp          M_mbp       1       num_PSVGA_mbp   num_PSVGA_mbp      num_PA_mbp};
num_comps_dmba =        {num_BBU_ops_dmba    M_dmba         M_dmba      1       0               0                  num_PA_dmba};
num_comps_hfc =         {num_BBU_ops_hfc     M_hfc          M_hfc       1       num_PSVGA_hfc   num_PSVGA_hfc      num_PA_hfc};

%% Step 3: Define type and power of components

% BBU power consumption per operation
% Theory from Despoisse (2020)[55] Yan (2019)
efficiency_BBU = 25e6;              % operations/s/mW
pw_BBU_per_op = bandwidths(case_num)/efficiency_BBU;      % mW/operation


% Analog elements power consumption per element
pw_per_mxLO =   140;                % mW,   based on Yan (2019), including buffers and ref. clock
pw_per_rfamp = [10 20 40];          % mW,   for various gain levels and 15% PAE, also input to loss_budget_x()
pw_per_ps = 10;                     % mW,   based on Yan (2019)
pw_per_vga = 15;                    % mW,   based on NXP doc and 9 dB gain, CMOS


% DAC power consumption
enob = 8;                       % bits,     min. for chosen SFDR, see DAC component analysis
Vdd = 2.7;                      % V,        based on DAC component analysis
f_cor_range = [0.5 0.8 1]*1e9;  % GHz,      based on DAC component analysis, function of BW        
f_cor = f_cor_range(case_num);  % GHz,      set f_cor value
OS_factor_dac = 2;              % for PAPR reduction


% PA power consumption
PAtech = PAtech_name;  % from user input
specs = 'm';           % specs: m = this model, d = despoisse2020 (benchmark)
category = '0';        % category: 0 = 'mean', otherwise 'min' or 'max'


% component power consumption in mW
% Order according to 'comp_names'
% input PA/DAC parameters according to their definition
comp_power = {pw_BBU_per_op, ...                                  % BBU
              [Vdd, enob, bandwidths(case_num), f_cor, OS_factor_dac],... % DAC
              pw_per_mxLO,...                                     % mxLO
              pw_per_rfamp,...                                    % RF amps
              pw_per_ps,...                                       % PS
              pw_per_vga, ...                                     % VGA
              [EIRP Gtx_antenna N_user PAtech specs category, ... % PA
              ]};   
            

%% Step 4: Compute total power 
% all outputs in mW, gets power_1dB

% MBPAA
[mbpaa_P_tot, mbp_P_tot_breakdown] = get_total_power_fc_ah(num_comps_mbp, comp_power, Ntx_range, "MBPAA");     

% DMBA
[dmba_P_tot, dmba_P_tot_breakdown] = get_total_power_dmba(num_comps_dmba, comp_power, Ntx_range);     

% HFC
[hfc_P_tot, hfc_P_tot_breakdown] = get_total_power_fc_ah(num_comps_hfc, comp_power, Ntx_range, "HFC");

%% Plotting
%% (A) Plot power breakdown

% y-axis upper bound: 'default' or custom (e.g. 100)
yl = "default";    
% yl = 100;         

% MBPAA 
plt = plotSingleBarsPdc(mbp_P_tot_breakdown, mbpaa_P_tot, comp_names, Ntx_range, N_user, yl);
%title({"Power consumption breakdown: MBPAA architecture", "Case "+case_num+", "+N_user+" users, tech: "+PAtech}, 'interpreter','none');
title("MBPAA")

% DMBA
plt = plotSingleBarsPdc(dmba_P_tot_breakdown, dmba_P_tot, comp_names, Ntx_range, N_user, yl);
%title({"Power consumption breakdown: DMBA architecture", "Case "+case_num+", "+N_user+" users, tech: "+PAtech}, 'interpreter','none');
title("DMBA")

% HFC
plt = plotSingleBarsPdc(hfc_P_tot_breakdown, hfc_P_tot, comp_names, Ntx_range, N_user, yl);
%title({"Power consumption breakdown: HFC architecture", "Case "+case_num+", "+N_user+" users, tech: "+PAtech}, 'interpreter','none');
title("HFC")

%% (B) Plot total system power consumption - comparision between architectures
% System total power consumption P_dc (W) vs number of antennas Ntx (log scale)
% Additional to plots in main paper

for makefig = 1
    figure; hold on;
    plot(Ntx_range, mbpaa_P_tot/1000, '-o', 'DisplayName', arch_names_fc{1}, 'Color',linecolors2(1,:)); 
    plot(Ntx_range, dmba_P_tot/1000, '-o', 'DisplayName', arch_names_fc{2}, 'Color',linecolors2(2,:)); 
    plot(Ntx_range, hfc_P_tot/1000, '-o', 'DisplayName', arch_names_fc{3}, 'Color',linecolors2(3,:)); 
    hold off; 
    xlim([0,520]); %ylim([0 450]);
    xticks(Ntx_range); xticklabels({'16','32','64','128','256','512'}); 
    xlabel('Total number of antennas'); ylabel("Power Comsumption[W]");
    grid on; legend('Location','northeastoutside');
    if specs == 'd' | category ~= 0
        title({"Total power consumption: Case "+case_num+", "+N_user+" users", ...
        "PA Tech: "+PAtech+",  model: '"+specs+"' ("+category+")"}, 'interpreter','none');
    else
        title({"Total power consumption: Case "+case_num+", "+N_user+" users", ...
        "PA Tech: "+PAtech}, 'interpreter','none');
    end
end

% <<< end of script >>>