%% ----------- INTRODUCTION -----------  %%
% Author: Natasha Birari, natashabirari@gmail.com, Aug 2022
%
% This script executes the base model scripts for fully connected and
% subarray architectures, and plots the system P_DC optima (value and
% corresponding optimum number of Ntx)
%
% The data must be created and saved manually once, especially the 75% LOS one
% Afterwards it can be simply loaded and plotted.
%
% The prompt prevents the data from being created accidentally if the
% script is Run directly.
%
% This script currently loads the following pre-saved data:
% Use Case 1: EIRP 40 dBm, EIRP 32 dBm
% Use Case 2: EIRP 40 dBm
% Use Case 3: EIRP 40 dBm, EIRP 37 dBm, EIRP 32 dBm (75% LOS)
% All data was saved for 3 PA technologies: CMOS SOI, GaAs, GaN
% Last saved: 30 May 2023
%
% The script can plot the following information with the loaded data:
% (1) Optimal power consumption and corresponding optimal array size
% (2) Optimal power consumption per user
% A list of plots per Use Case is provided later below.
%
% NOTE: The user input parameters in the base model script ARE OVERWRITTEN
% by the inputs defined in this file.
%
% This script works together with other functions for specific calculations
% (see list below)
%
% ---------     Dependent functions     ---------
% This script uses the following functions:
% main_model_fullyconnected.m   system power consumption MBPAA, DMBA, HFC
% main_model_subarray.m         system power consumption SDMBA, HSA

%% Input prompt
create_data = input('To create the data, choose yes. To simply load, choose no. [y/n]: ','s');

if create_data == 'y'
    % Set input params and run model's base files
    
    % Choose use case
    % Overwrites the param in the base file
    case_num_run = 3;
    
    % Set number of simultaneous users 
    % Overwrites the param in the base file when main_run is run
    N_user_range_run = [2, 4, 8];  % Possible range of values
    
    % 1 if NLOS path is checked, 0 for LOS only
    % Overwrites the param in the base file when main_run is run
    isNLOS_run = 1;
    LOS_ratio_default_run = 0.75;
    
    % EIRP toggle
    % '0' for default 40 dBm EIRP, '1' for calculated average EIRP which
    % considers LOS probability
    % Overwrites the param in the base file when main_run is run
    EIRP_toggle_run = 0;
    EIRP_default_run = 32; % Manually set EIRP
    
    
    % Choose PA techs to plot
    % Overwrites the param in the base file
    PAtechs_run = ["CMOS_SOI","GaAs","GaN"];
    
    
    % Pre-allocate 
    saved_data = [];    % P_dc data
    opt = [];           % optimum value
    loc = [];           % location of optimum
    
    % Loop 
    for u_idx = 1:size(N_user_range_run, 2)             % loop over num. users
        N_user_run = N_user_range_run(u_idx); 
        for tech_idx = 1:size(PAtechs_run, 2)           % loop over PA techs
            PAtech_run = PAtechs_run(tech_idx);
            
            % Hide plots generated in base model scripts
            set(0,'DefaultFigureVisible','off');
            
            % Generate power consumption data
            main_model_fullyconnected;  % run base model script
            main_model_subarray;        % run base model script
            fprintf("Finished: Case number: "+case_num+", N_user: "+N_user+", PA tech: "+PAtech+", EIRP: "+EIRP+"\n");

            % Close hidden plots
            figHandles = findobj('Type','figure','Visible','off');
            close(figHandles);

            % Save generated data
            saved_data(1:5,:) = [mbpaa_P_tot; dmba_P_tot; hfc_P_tot; sdmba_P_tot_vec; hsa_P_tot_vec];

            for arch_idx = 1:5
                [a,b] = min(saved_data(arch_idx,:));

                if isnan(a)
                    b = NaN;
                end

                opt(arch_idx, tech_idx, u_idx) = a;
                loc(arch_idx, tech_idx, u_idx) = b;
            end
        end
    end

    % Clear temporary variables
    clear('create_data', 'u_idx','tech_idx', 'arch');
    
    % save data to load directly in the future. File name key:
    % optima_valEIRP_casenumber_(LOS%ifrelevant)_numusers_numtechs
    if isNLOS_run == 0
        fname = sprintf('optima_%deirp_case%d_allu_3tech.mat', round(EIRP), case_num);
    elseif isNLOS_run == 1
        fname = sprintf('optima_%deirp_case%d_los%d_allu_3tech.mat', floor(EIRP), case_num, floor(100*LOS_ratio));
    end
    save(fname,'opt','loc');

    % Reset figure visibility
    set(0,'DefaultFigureVisible','on');
else
    %% manually load from previously saved files
    % Define files
    
    % EIRP 40 dBm, cases 1 - 3
    opt_40_c1 = load('saved_data/optima_40eirp_case1_allu_3tech.mat', 'opt').('opt');
    loc_40_c1 = load('saved_data/optima_40eirp_case1_allu_3tech.mat', 'loc').('loc');
    opt_40_c2 = load('saved_data/optima_40eirp_case2_allu_3tech.mat', 'opt').('opt');
    loc_40_c2 = load('saved_data/optima_40eirp_case2_allu_3tech.mat', 'loc').('loc');
    opt_40_c3 = load('saved_data/optima_40eirp_case3_allu_3tech.mat', 'opt').('opt');
    loc_40_c3 = load('saved_data/optima_40eirp_case3_allu_3tech.mat', 'loc').('loc');

    % EIRP 32 dBm, case 1
    % Note: for 2u --> EIRP = 30.61dBm
    opt_32_c1 = load('saved_data/optima_32eirp_case1_allu_3tech.mat', 'opt').('opt');
    loc_32_c1 = load('saved_data/optima_32eirp_case1_allu_3tech.mat', 'loc').('loc');

    % EIRP 37 dBm, case 3
    % Note: for 2u --> EIRP = 35.72dBm
    opt_37_c3 = load('saved_data/optima_37eirp_case3_allu_3tech.mat', 'opt').('opt');
    loc_37_c3 = load('saved_data/optima_37eirp_case3_allu_3tech.mat', 'loc').('loc');

    % EIRP 32 dBm, case 3, 75% LOS
    % Note: for 2u --> EIRP = 15.71dBm
    opt_32_c3_los75 = load('saved_data/optima_32eirp_case3_los75_allu_3tech.mat', 'opt').('opt');
    loc_32_c3_los75 = load('saved_data/optima_32eirp_case3_los75_allu_3tech.mat', 'loc').('loc');
    
    % clear temp variables
    clear('create_data');

end

%% OPTIMAL POWER - NORMALISED PER USER PLOTS
% Get per-user power consumption of the system by normalising the data
% pu --> short for per user
% NOTE: this code is only accurate for the pre-saved data values and
% values of N_user_range = [2, 4, 8, 16, 32]
% Changes to this need to be incorporated with care by changing the
% normalisation formula

for get_pu = 1
    
    opt_40_c1_pu = opt_40_c1;
    for idx_nm = 1:size(opt_40_c1_pu,3)
        opt_40_c1_pu(:,:,idx_nm) = opt_40_c1_pu(:,:,idx_nm)./N_user_range(idx_nm);
    end
    
    opt_32_c1_pu = opt_32_c1;
    for idx_nm = 1:1:size(opt_32_c1_pu,3)
    opt_32_c1_pu(:,:,idx_nm) = opt_32_c1_pu(:,:,idx_nm)./N_user_range(idx_nm);
    end

    opt_40_c3_pu = opt_40_c3;
    for idx_nm = 1:1:size(opt_40_c3_pu,3)
        opt_40_c3_pu(:,:,idx_nm) = opt_40_c3_pu(:,:,idx_nm)./N_user_range(idx_nm);
    end
    
    opt_32_c3_los75_pu = opt_32_c3_los75;
    for idx_nm = 1:1:size(opt_40_c3_pu,3)
        opt_32_c3_los75_pu(:,:,idx_nm) = opt_32_c3_los75_pu(:,:,idx_nm)./N_user_range(idx_nm);
    end

    opt_40_c2_pu = opt_40_c2;
    for idx_nm = 1:1:size(opt_40_c2_pu,3)
        opt_40_c2_pu(:,:,idx_nm) = opt_40_c2_pu(:,:,idx_nm)./N_user_range(idx_nm);
    end

end
%% Plot settings

% Define x axis
uxx = 1:1:15;

% Define colours
clrs=[      0    0.4471    0.7412; ...          % blue
       0.8510    0.3255    0.0980; ...          % red
       0.9294    0.6941    0.1255; ...          % yellow
       0.4941    0.1843    0.5569; ...          % purple
       0.3922    0.8314    0.0745]; ...         % green

% Define lines and markers
markerVec = {'o','*','v'};
% markerVec_new = {'o','*','^'};
linVec_plot = {'-','--','-.','--','-','-'}; 
markerFaceClr = {repmat(['none'],5,1),repmat(['none'],5,1),clrs};
markerSz = {30 50 30};

%% CASE 1

% The following plots are generated:
% 1_1. Optimal P_TOT and optimal array size - EIRP 40 dBm
% 1_2. Optimal P_TOT per user - EIRP 40 dBm
% 1_3. Optimal P_TOT and optimal array size - EIRP 32 dBm
% 1_4. Optimal P_TOT per user - EIRP 32 dBm

%% Figure 1_1: Optimal P_TOT and array size: Case 1, EIRP 40 dBm
% Optimal P_TOT for varying array size (Ntx) and architectures, 
% grouped by PA technology

% Load plot data for optimal P_TOT
data_plot = opt_40_c1;
for makefig = 1
figure; hold on;
for tech_idx = 1:3                                      % loop over PA tech
    for arch_idx = 1:5                                  % loop over archs
    u_idx = size(data_plot,3);                          % choose users
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);        % get data as 2D mat
    dat = squeeze(dat);
    
    % Plot
    plot(uxx(1:u_idx), dat/1e3, 'LineStyle',linVec_plot{tech_idx},'Color',clrs(arch_idx,:),...
        'LineWidth',0.8,'Marker',markerVec{tech_idx},'MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:));
    
    end
end
% Set axis properties
ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
xticks(1:1:u_idx); xticklabels(num2str(N_user_range(1:u_idx)'));
ylabel('P_{TOT}, [W]'); xlabel('Number of users');
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"];%+[" CMOS"; " GaAs"; " GaN"];
legText = repmat(legText,3,1);
legend(reshape(legText', 1, []), 'Location','southoutside', 'NumColumns', 3);
grid on; box on;
end
title('Optimal P_{TOT} (EIRP = 40 dBm, Case 1)');

% Load plot data for array size
data_plot = loc_40_c1;
for makefig = 1
% relative horizontal spacing - set manually (for now)
% size of matrices below: nuser x arch
tech1 = [    0     1     1     0     3;
             0     1     1     0     2;
             0     1     2     0     0;
             0     1     2     0     0;
             0     1     2     3     0];
tech2 = [    0     0     2     0     3;
             0     1     2     0     2;
             0     1     2     0     1;
             0     1     2     0     0;
             0     1     1     3     0];
tech3 = [    0     0     2     0     3;
             0     1     2     0     2;
             0     1     2     0     1;
             0     1     0     3     0;
             0     1     0     3     0];

sph = {tech1' tech2' tech3'};
% relative vertical spacing
% arch x Ntx x users
sp = zeros([size(data_plot,1) length(Ntx_range) size(data_plot,3)]);

for u_idx = 1:size(sp,3)             % loop over num. users
    for archs = 1:size(sp,1)         % loop over archs
        for nn = 1:size(sp,2)        % loop over Ntx value
        sp(archs, nn, u_idx) = nnz(data_plot(archs, :, u_idx) == nn);
        end
    end
end

dyy = []; % preallocate

figure; hold on;
for tech_idx = 1:3                  % loop over PA techs
    for arch_idx = 1:size(sp,1)      % loop over archs
    u_idx = size(sp,3);             % choose users
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);    % get data as 2D mat
    dat = squeeze(dat);
    
    % for equidistant scatter points - vertical
    for ii = 1:u_idx
    if ~isnan(dat(ii))
        if sp(arch_idx,dat(ii),ii) == 3                       % tech 1,2 and 3
            dyy(ii,:) = -[-0.2 0 0.2];
        elseif sp(arch_idx,dat(ii),ii) == 2
            if loc_40_c1(arch_idx,1,ii) == loc_40_c1(arch_idx,2,ii) % tech 1 and 2
                dyy(ii,:) = -[-0.2 0.2 0]./2;
            elseif loc_40_c1(arch_idx,1,ii) == loc_40_c1(arch_idx,3,ii) % tech 2 and 3
                dyy(ii,:) = -[-0.2 0 0.2]./2;
            else                                            % tech 1 and 3
                dyy(ii,:) = -[0 -0.2 0.2]./2;               
            end
        else
            dyy(ii,:) = [0 0 0];                        % 0 or 1 tech
        end
    end
    end
    
    % for equidistant scatter points - horizontal
    dxx = sph{tech_idx}(arch_idx,:).*0.1;
     
    xoff = 0.2;
   
    scatter(uxx(1:u_idx)+xoff+dxx, dat + dyy(:,tech_idx), markerSz{tech_idx}, clrs(arch_idx,:), 'LineWidth', 0.8, ...
        'Marker',markerVec{tech_idx},'MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:),'MarkerEdgeColor',clrs(arch_idx,:));
    end
end
ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
xticks(1:1:u_idx); 
xticklabels(num2str(N_user_range(1:u_idx)')); xlim([0.5, u_idx(end)+1]);
% %xticks = [];
yticks(0:1:6); 
yticklabels(['  0'; num2str(Ntx_range');'520']); 
ylim([0 6.5]);
ylabel('N_{TX}'); xlabel('Number of users');
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"];%+[" CMOS"; " GaAs"; " GaN"];
legText = repmat(legText,3,1);
legend(reshape(legText', 1, []), 'Location','southoutside', 'NumColumns', 3);
grid on;
box on;
set(gca,'YMinorTick','on');
ax.YAxis.MinorTickValues = [0.5:1:6.5]; %grid minor;
%ax.YRuler.MinorTick = [0.1:0.1:0.9];
end
title('Optimal array size (EIRP = 40 dBm, Case 1)');

%% Figure 1_2: Optimal P_TOT per user: Case 1, EIRP 40 dBm
% Optimal P_TOT per user for varying number of users and architectures, 
% grouped by PA technology

% Load plot data for optimal P_TOT per user
data_plot = opt_40_c1_pu;
for makefig = 1
figure; t = tiledlayout(1,3);
for tech_idx = 1:3                        % loop over PA tech
    nexttile; hold on;
    for arch_idx = 1:5                   % loop over archs
        u_idx = size(data_plot,3);          % choose users
        dat = data_plot(arch_idx, tech_idx, 1:u_idx);     % get data as 2D mat
        dat = squeeze(dat);

        % plot
        plot(uxx(1:u_idx), (dat/1e3), 'LineStyle',linVec_plot{1},'Color',clrs(arch_idx,:),...
            'LineWidth',0.8,'Marker','.','MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:));

    end
% Set axis properties
ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
ax.FontSize = 10;
xticks(1:1:u_idx); xticklabels(num2str(N_user_range(1:u_idx)'));
xlim([1 u_idx]);
ylabel('Optimal P_{TOT} per user [W]'); xlabel('Number of users');
%legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"]+[" CMOS"; " GaAs"; " GaN"];
PAtechs_fig = [" CMOS"; " GaAs"; " GaN"];
grid on; box on;
ylim([0 12]);
title(PAtechs_fig(tech_idx), 'interpreter','none');

if tech_idx == 1
    legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"];
legend(reshape(legText', 1, []), 'Location','eastoutside', 'NumColumns', 1);
ax.FontSize = 11;
end
end
end
t.Padding = 'none';
t.TileSpacing = 'none';
sgtitle('Optimal P_{TOT} per user (EIRP = 40 dBm, Case 1)');

%% Figure 1_3: Optimal P_TOT and array size: Case 1, EIRP 32 dBm
% Optimal P_TOT for varying array size (Ntx) and architectures,

% Load plot data for optimal P_TOT
data_plot = opt_32_c1;
for makefig = 1
figure; hold on;
for tech_idx = 1:3                        % tech
    for arch_idx = 1:5                    % arch
    u_idx = size(data_plot,3);
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);     % users
    dat = squeeze(dat);
    plot(uxx(1:u_idx), dat/1e3, 'LineStyle',linVec_plot{tech_idx},'Color',clrs(arch_idx,:),...
        'LineWidth',0.8,'Marker',markerVec{tech_idx},'MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:));
    end
end
ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
xticks(1:1:u_idx); xticklabels(num2str(N_user_range(1:u_idx)'));
ylabel('P_{TOT}, [W]'); xlabel('Number of users');
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"];%+[" CMOS"; " GaAs"; " GaN"];
legText = repmat(legText,3,1);
legend(reshape(legText', 1, []), 'Location','southoutside', 'NumColumns', 3);
grid on; box on;
end
title('Optimal P_{TOT} (EIRP = 32.36 dBm, Case 1)');

% Load plot data for optimal array size
data_plot = loc_32_c1;
for makefig = 1
% relative horizontal spacing - manual for now
% nuser x arch
tech1 = [    0     1     1     0     0;
             0     1     2     0     0;
             0     1     2     0     0;
             0     0     2     0     0;
             0     1     2     0     0];
tech2 = [    0     1     2     0     2;
             0     0     2     0     0;
             0     1     2     3     0;
             0     1     2     1     0;
             0     1     1     0     0];
tech3 = [    0     1     2     3     2;
             0     0     2     1     0;
             0     1     2     3     0;
             0     1     2     1     0;
             0     1     1     0     1];

sph = {tech1' tech2' tech3'};
% relative vertical spacing
% arch x Ntx x users
sp = zeros([size(data_plot,1) length(Ntx_range) size(data_plot,3)]);
for u_idx = 1:size(sp,3)                % num. users value
    for archs = 1:size(sp,1)         % arch type
            for nn = 1:size(sp,2)    % Ntx value
            sp(archs,nn,u_idx) = nnz(data_plot(archs,:,u_idx) == nn);
            end
    end
end

dyy = [];

figure; hold on;
for tech_idx = 1:3                        % loop over PA tecnologies
    for arch_idx = 1:size(sp,1)              % loop over architectures
    u_idx = size(sp,3);             % choose users
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);    % get data as 2D mat
    dat = squeeze(dat);
    
    % for equidistant scatter points - vertical
    for ii = 1:u_idx
    if ~isnan(dat(ii))
        if sp(arch_idx,dat(ii),ii) == 3                       % tech 1,2 and 3
            dyy(ii,:) = -[-0.2 0 0.2];
        elseif sp(arch_idx,dat(ii),ii) == 2
            if loc_40_c1(arch_idx,1,ii) == loc_40_c1(arch_idx,2,ii) % tech 1 and 2
                dyy(ii,:) = -[-0.2 0.2 0]./2;
            elseif loc_40_c1(arch_idx,1,ii) == loc_40_c1(arch_idx,3,ii) % tech 1 and 3
                dyy(ii,:) = [-0.2 0 0.2]./2;
            else                                            % tech 2 and 3
                dyy(ii,:) = -[0 -0.2 0.2]./2;               
            end
        else
            dyy(ii,:) = [0 0 0];                        % 0 or 1 tech
        end
    end
    end
    
    % for equidistant scatter points - horizontal
    dxx = sph{tech_idx}(arch_idx,:).*0.1;
     
    xoff = 0.2;
   
    scatter(uxx(1:u_idx)+xoff+dxx, dat + dyy(:,tech_idx), markerSz{tech_idx}, clrs(arch_idx,:), 'LineWidth', 0.8, ...
        'Marker',markerVec{tech_idx},'MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:),'MarkerEdgeColor',clrs(arch_idx,:));
    end
end
ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
xticks(1:1:u_idx); 
xticklabels(num2str(N_user_range(1:u_idx)')); xlim([0.5, u_idx(end)+1]);
% %xticks = [];
yticks(0:1:6); 
yticklabels(['  0'; num2str(Ntx_range');'520']); 
ylim([0 6.5]);
ylabel('N_{TX}'); xlabel('Number of users');
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"];%+[" CMOS"; " GaAs"; " GaN"];
legText = repmat(legText,3,1);
legend(reshape(legText', 1, []), 'Location','southoutside', 'NumColumns', 3);
grid on;
box on;
set(gca,'YMinorTick','on');
ax.YAxis.MinorTickValues = [0.5:1:6.5]; %grid minor;
%ax.YRuler.MinorTick = [0.1:0.1:0.9];
end
title('Optimal array size (EIRP = 32.36 dBm, Case 1)');

%% Figure 1_4: Optimal P_TOT per user: Case 1, EIRP 32 dBm
% Optimal P_TOT per user for varying number of users and architectures, 
% grouped by PA technology

% Load plot data for optimal P_TOT per user
data_plot = opt_32_c1_pu;
for makefig = 1
figure; tiledlayout(1,3);
for tech_idx = 1:3                        % tech
    nexttile; hold on;
    for arch_idx = 1:5                   % arch
    u_idx = size(data_plot,3);
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);     % users
    dat = squeeze(dat);
    plot(uxx(1:u_idx), dat/1e3, 'LineStyle',linVec_plot{1},'Color',clrs(arch_idx,:),...
        'LineWidth',0.8,'Marker','.','MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:));
    end

ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
ax.FontSize = 10;
xticks(1:1:u_idx); xticklabels(num2str(N_user_range(1:u_idx)'));
xlim([1 u_idx]);
ylabel('P_{TOT} per user [W]'); xlabel('Number of users');
%legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"]+[" CMOS"; " GaAs"; " GaN"];
PAtechs_figs = [" CMOS"; " GaAs"; " GaN"];
grid on; box on;
ylim([0 5]);
title(PAtechs_figs(tech_idx), 'interpreter','none');

end
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"];
legend(reshape(legText', 1, []), 'Location','eastoutside', 'NumColumns', 1);
end
sgtitle('Optimal P_{TOT} per user (EIRP = 32.36 dBm, Case 1)');

%% CASE 2

% The following plots are generated:
% 2_1. Optimal P_TOT and optimal array size - EIRP 40 dBm
% 2_2. Optimal P_TOT per user - EIRP 40 dBm

%% Figure 2_1: Optimal P_TOT and array size: Case 2, EIRP 40 dBm
% Optimal P_TOT for varying array size (Ntx) and architectures,
% grouped by PA tech

% Load plot data for optimal P_TOT
data_plot = opt_40_c2;
for makefig = 1
figure; hold on;
for tech_idx = 1:3                        % tech
    for arch_idx = 1:5                    % arch
    u_idx = size(data_plot,3);
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);     % users
    dat = squeeze(dat);
    plot(uxx(1:u_idx), dat/1e3, 'LineStyle',linVec_plot{tech_idx},'Color',clrs(arch_idx,:),...
        'LineWidth',0.8,'Marker',markerVec{tech_idx},'MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:));
    end
end
ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
xticks(1:1:u_idx); xticklabels(num2str(N_user_range(1:u_idx)'));
ylabel('P_{TOT} [W]'); xlabel('Number of users');
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"];%+[" CMOS"; " GaAs"; " GaN"];
legText = repmat(legText,3,1);
legend(reshape(legText', 1, []), 'Location','southoutside', 'NumColumns', 3);
grid on; box on;
end
title('Optimal P_{TOT} (EIRP = 40 dBm, Case 2)');

% Load plot data for optimal array size
data_plot = loc_40_c2;
for makefig = 1
% relative horizontal spacing - manual for now
% nuser x arch
tech1 = [    0     0     1     0     2;
             0     1     2     0     1;
             0     1     2     0     1;
             0     1     2     0     0];
tech2 = [    0     0     1     0     2;
             0     1     2     0     1;
             0     1     2     0     1;
             0     1     2     0     0];
tech3 = [    0     0     1     0     2;
             0     0     1     0     1;
             0     1     0     1     1;
             0     1     0     3     0];

sph = {tech1' tech2' tech3'};

% relative vertical spacing
% arch x Ntx x users
sp = zeros([size(data_plot,1) length(Ntx_range) size(data_plot,3)]);
for u_idx = 1:size(sp,3)                % num. users value
    for archs = 1:size(sp,1)         % arch type
            for nn = 1:size(sp,2)    % Ntx value
            sp(archs,nn,u_idx) = nnz(data_plot(archs,:,u_idx) == nn);
            end
    end
end

dyy = [];

figure; hold on;
for tech_idx = 1:3                        % techs
    for arch_idx = 1:size(sp,1)           % archs
    u_idx = size(sp,3);             % choose users
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);    % get data as 2D mat
    dat = squeeze(dat);
    
    % for equidistant scatter points - vertical
    for ii = 1:u_idx
    if ~isnan(dat(ii))
        if sp(arch_idx,dat(ii),ii) == 3                       % tech 1,2 and 3
            dyy(ii,:) = -[-0.2 0 0.2];
        elseif sp(arch_idx,dat(ii),ii) == 2
            if loc_40_c1(arch_idx,1,ii) == loc_40_c1(arch_idx,2,ii) % tech 1 and 2
                dyy(ii,:) = -[-0.2 0.2 0]./2;
            elseif loc_40_c1(arch_idx,1,ii) == loc_40_c1(arch_idx,3,ii) % tech 2 and 3
                dyy(ii,:) = -[-0.2 0 0.2]./2;
            else                                            % tech 1 and 3
                dyy(ii,:) = -[0 -0.2 0.2]./2;               
            end
        else
            dyy(ii,:) = [0 0 0];                        % 0 or 1 tech
        end
    end
    end
    
    % for equidistant scatter points - horizontal
    dxx = sph{tech_idx}(arch_idx,:).*0.1;
     
    xoff = 0.2;
   
    scatter(uxx(1:u_idx)+xoff+dxx, dat + dyy(:,tech_idx), markerSz{tech_idx}, clrs(arch_idx,:), 'LineWidth', 0.8, ...
        'Marker',markerVec{tech_idx},'MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:),'MarkerEdgeColor',clrs(arch_idx,:));
    end
end
ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
xticks(1:1:u_idx); 
xticklabels(num2str(N_user_range(1:u_idx)')); xlim([0.5, u_idx(end)+1]);
% %xticks = [];
yticks(0:1:6); 
yticklabels(['  0'; num2str(Ntx_range');'520']); 
ylim([0 6.5]);
ylabel('N_{TX}'); xlabel('Number of users');
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"];%+[" CMOS"; " GaAs"; " GaN"];
legText = repmat(legText,3,1);
legend(reshape(legText', 1, []), 'Location','southoutside', 'NumColumns', 3);
grid on;
box on;
set(gca,'YMinorTick','on');
ax.YAxis.MinorTickValues = [0.5:1:6.5]; %grid minor;
%ax.YRuler.MinorTick = [0.1:0.1:0.9];
end
title('Optimal array size (EIRP = 40 dBm, Case 2)');

%% Figure 2_2: Optimal P_TOT per user: Case 2, EIRP 40 dBm
% Optimal P_TOT per user for varying number of users and architectures, 
% grouped by PA technology

% Load plot data
data_plot = opt_40_c2_pu;

for makefig = 1
figure; t = tiledlayout(1,3);
for tech_idx = 1:3                        % tech
    nexttile; hold on;
    for arch_idx = 1:5                   % arch
    u_idx = size(data_plot,3);
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);     % users
    dat = squeeze(dat);
    plot(uxx(1:u_idx), dat/1e3, 'LineStyle','-','Color',clrs(arch_idx,:),...
        'LineWidth',0.8,'Marker','.','MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:));
    end

ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
xticks(1:1:u_idx); xticklabels(num2str(N_user_range(1:u_idx)'));
ylabel('Optimal P_{TOT} per user, [W]'); xlabel('Number of users');
%legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"]+[" CMOS"; " GaAs"; " GaN"];
%legText = [" CMOS"; " GaAs"; " GaN"];
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"];
legend(reshape(legText', 1, []), 'Location','northeast', 'NumColumns', 1);
grid on; box on;
ylim([0 12]);
ax.FontSize = 11;
title(PAtechs_figs(tech_idx));
end
end
t.Padding = 'none';
t.TileSpacing = 'none';
sgtitle('Optimal P_{TOT} per user (EIRP = 40 dBm, Case 2)');

%% CASE 3

% The following plots are generated:
% 3_1. Optimal P_TOT and optimal array size - EIRP 40 dBm
% 3_2. Optimal P_TOT per user - EIRP 40 dBm
% 3_3. Optimal P_TOT and optimal array size - EIRP 37 dBm

%% Figure 3_1: Optimal P_TOT and array size: Case 2, EIRP 40 dBm
% Optimal P_TOT for varying array size (Ntx) and architectures,
% grouped by PA tech

% Load plot data for optimal P_TOT
data_plot = opt_40_c3;
for makefig = 1
figure; hold on;
for tech_idx = 1:3                        % tech
    for arch_idx = 1:5                    % arch
    u_idx = size(data_plot,3);
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);     % users
    dat = squeeze(dat);
    plot(uxx(1:u_idx), dat/1e3, 'LineStyle',linVec_plot{tech_idx},'Color',clrs(arch_idx,:),...
        'LineWidth',0.8,'Marker',markerVec{tech_idx},'MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:));
    end
end
ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
xticks(1:1:u_idx); xticklabels(num2str(N_user_range(1:u_idx)'));
ylabel('P_{TOT} [W]'); xlabel('Number of users');
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"];%+[" CMOS"; " GaAs"; " GaN"];
legText = repmat(legText,3,1);
legend(reshape(legText', 1, []), 'Location','southoutside', 'NumColumns', 3);
grid on; box on;
end
title('Optimal P_{TOT} (EIRP = 40 dBm, Case 3)');

% Load plot data for optimal array size
data_plot = loc_40_c3;
for makefig = 1
% relative horizontal spacing - manual for now
% nuser x arch
tech1 = [    0     0     1     0     2;
             0     1     2     0     1;
             0     1     2     0     0];
tech2 = [    0     0     1     0     2;
             0     0     2     0     1;
             0     1     2     0     1];
tech3 = [    0     0     1     0     2;
             0     0     1     0     1;
             0     1     0     1     1];

sph = {tech1' tech2' tech3'};

% relative vertical spacing
% arch x Ntx x users
sp = zeros([size(data_plot,1) length(Ntx_range) size(data_plot,3)]);
for u_idx = 1:size(sp,3)                % num. users value
    for archs = 1:size(sp,1)         % arch type
            for nn = 1:size(sp,2)    % Ntx value
            sp(archs,nn,u_idx) = nnz(data_plot(archs,:,u_idx) == nn);
            end
    end
end

dyy = [];

figure; hold on;
for tech_idx = 1:3                        % techs
    for arch_idx = 1:size(sp,1)           % archs
    u_idx = size(sp,3);             % choose users
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);    % get data as 2D mat
    dat = squeeze(dat);
    
    % for equidistant scatter points - vertical
    for ii = 1:u_idx
    if ~isnan(dat(ii))
        if sp(arch_idx,dat(ii),ii) == 3                       % tech 1,2 and 3
            dyy(ii,:) = -[-0.2 0 0.2];
        elseif sp(arch_idx,dat(ii),ii) == 2
            if loc_40_c1(arch_idx,1,ii) == loc_40_c1(arch_idx,2,ii) % tech 1 and 2
                dyy(ii,:) = -[-0.2 0.2 0]./2;
            elseif loc_40_c1(arch_idx,1,ii) == loc_40_c1(arch_idx,3,ii) % tech 2 and 3
                dyy(ii,:) = -[-0.2 0 0.2]./2;
            else                                            % tech 1 and 3
                dyy(ii,:) = -[0 -0.2 0.2]./2;               
            end
        else
            dyy(ii,:) = [0 0 0];                        % 0 or 1 tech
        end
    end
    end
    
    % for equidistant scatter points - horizontal
    dxx = sph{tech_idx}(arch_idx,:).*0.1;
     
    xoff = 0.2;
   
    scatter(uxx(1:u_idx)+xoff+dxx, dat + dyy(:,tech_idx), markerSz{tech_idx}, clrs(arch_idx,:), 'LineWidth', 0.8, ...
        'Marker',markerVec{tech_idx},'MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:),'MarkerEdgeColor',clrs(arch_idx,:));
    end
end
ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
xticks(1:1:u_idx); 
xticklabels(num2str(N_user_range(1:u_idx)')); xlim([0.5, u_idx(end)+1]);
% %xticks = [];
yticks(0:1:6); 
yticklabels(['  0'; num2str(Ntx_range');'520']); 
ylim([0 6.5]);
ylabel('N_{TX}'); xlabel('Number of users');
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"];%+[" CMOS"; " GaAs"; " GaN"];
legText = repmat(legText,3,1);
legend(reshape(legText', 1, []), 'Location','southoutside', 'NumColumns', 3);
grid on;
box on;
set(gca,'YMinorTick','on');
ax.YAxis.MinorTickValues = [0.5:1:6.5]; %grid minor;
%ax.YRuler.MinorTick = [0.1:0.1:0.9];
end
title('Optimal array size (EIRP = 40 dBm, Case 3)');

%% Figure 3_2: Optimal P_TOT per user: Case 3, EIRP 40 dBm
% Optimal P_TOT per user for varying number of users and architectures, 
% grouped by PA technology

% Load plot data
data_plot = opt_40_c3_pu;
for makefig = 1
figure; t = tiledlayout(1,3);
for tech_idx = 1:3                        % tech
    nexttile; hold on;
    for arch_idx = 1:5                   % arch
    u_idx = size(data_plot,3);
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);     % users
    dat = squeeze(dat);
    plot(uxx(1:u_idx), dat/1e3, 'LineStyle','-','Color',clrs(arch_idx,:),...
        'LineWidth',0.8,'Marker','.','MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:));
    end

ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
xticks(1:1:u_idx); xticklabels(num2str(N_user_range(1:u_idx)'));
ylabel('Optimal P_{TOT} per user, [W]'); xlabel('Number of users');
%legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"]+[" CMOS"; " GaAs"; " GaN"];
PAtechs_figs = [" CMOS SOI"; " GaAs"; " GaN"];
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"];
legend(reshape(legText', 1, []), 'Location','best', 'NumColumns', 1);
grid on; box on;
ylim([0 20]);
ax.FontSize = 11;
title(PAtechs_figs(tech_idx));
end
end
t.Padding = 'none';
t.TileSpacing = 'none';
sgtitle('Optimal P_{TOT} per user (EIRP = 40 dBm, Case 3)');

%% Figure 3_3: Optimal P_TOT per user: Case 3, EIRP 37 dBm
% Optimal P_TOT for varying array size (Ntx) and architectures,
% grouped by PA tech

% Load plot data for optimal P_TOT
data_plot = opt_37_c3;
for makefig = 1
figure; hold on;
for tech_idx = 1:3                        % tech
    for arch_idx = 1:5                    % arch
    u_idx = size(data_plot,3);
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);     % users
    dat = squeeze(dat);
    plot(uxx(1:u_idx), dat/1e3, 'LineStyle',linVec_plot{tech_idx},'Color',clrs(arch_idx,:),...
        'LineWidth',0.8,'Marker',markerVec{tech_idx},'MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:));
    end
end
ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
xticks(1:1:u_idx); xticklabels(num2str(N_user_range(1:u_idx)'));
ylabel('P_{TOT} [W]'); xlabel('Number of users');
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"];%+[" CMOS"; " GaAs"; " GaN"];
legText = repmat(legText,3,1);
legend(reshape(legText', 1, []), 'Location','southoutside', 'NumColumns', 3);
grid on; box on;
end
title('Optimal P_{TOT} (EIRP = 37.5 dBm, Case 3)');

% Load plot data for optimal array size
data_plot = loc_37_c3;
for makefig = 1
% relative horizontal spacing - manual for now
% nuser x arch
tech1 = [    0     1     2     1     0;
             0     1     2     1     0;
             0     1     2     3     0];
tech2 = [    0     0     2     1     0;
             0     1     2     1     1;
             0     1     2     3     0];
tech3 = [    0     0     2     1     0;
             0     1     2     1     1;
             0     1     2     3     0];

sph = {tech1' tech2' tech3'};
% relative vertical spacing
% arch x Ntx x users
sp = zeros([size(data_plot,1) length(Ntx_range) size(data_plot,3)]);
for u_idx = 1:size(sp,3)                % num. users value
    for archs = 1:size(sp,1)         % arch type
            for nn = 1:size(sp,2)    % Ntx value
            sp(archs,nn,u_idx) = nnz(data_plot(archs,:,u_idx) == nn);
            end
    end
end

dyy = [];

figure; hold on;
for tech_idx = 1:3                        % techs
    for arch_idx = 1:size(sp,1)           % archs
    u_idx = size(sp,3);             % choose users
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);    % get data as 2D mat
    dat = squeeze(dat);
    
    % for equidistant scatter points - vertical
    for ii = 1:u_idx
    if ~isnan(dat(ii))
        if sp(arch_idx,dat(ii),ii) == 3                       % tech 1,2 and 3
            dyy(ii,:) = -[-0.2 0 0.2];
        elseif sp(arch_idx,dat(ii),ii) == 2
            if loc_40_c1(arch_idx,1,ii) == loc_40_c1(arch_idx,2,ii) % tech 1 and 2
                dyy(ii,:) = -[-0.2 0.2 0]./2;
            elseif loc_40_c1(arch_idx,1,ii) == loc_40_c1(arch_idx,3,ii) % tech 1 and 3
                dyy(ii,:) = [-0.2 0 0.2]./2;
            else                                            % tech 2 and 3
                dyy(ii,:) = -[0 -0.2 0.2]./2;               
            end
        else
            dyy(ii,:) = [0 0 0];                        % 0 or 1 tech
        end
    end
    end
    
    % for equidistant scatter points - horizontal
    dxx = sph{tech_idx}(arch_idx,:).*0.1;
     
    xoff = 0.2;
   
    scatter(uxx(1:u_idx)+xoff+dxx, dat + dyy(:,tech_idx), markerSz{tech_idx}, clrs(arch_idx,:), 'LineWidth', 0.8, ...
        'Marker',markerVec{tech_idx},'MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:),'MarkerEdgeColor',clrs(arch_idx,:));
    end
end
ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
xticks(1:1:u_idx); 
xticklabels(num2str(N_user_range(1:u_idx)')); xlim([0.5, u_idx(end)+1]);
% %xticks = [];
yticks(0:1:6); 
yticklabels(['  0'; num2str(Ntx_range');'520']); 
ylim([0 6.5]);
ylabel('N_{TX}'); xlabel('Number of users');
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"];%+[" CMOS"; " GaAs"; " GaN"];
legText = repmat(legText,3,1);
legend(reshape(legText', 1, []), 'Location','southoutside', 'NumColumns', 3);
grid on;
box on;
set(gca,'YMinorTick','on');
ax.YAxis.MinorTickValues = [0.5:1:6.5]; %grid minor;
%ax.YRuler.MinorTick = [0.1:0.1:0.9];
end
title('Optimal array size (EIRP = 37.5 dBm, Case 3)');

%% For NLOS scenarios:

%% Figure 4_1: Optimal P_TOT and array size: Case 3, EIRP 32 dBm, LOS 75%
% Optimal P_TOT for varying array size (Ntx) and architectures in NLOS
% scenario, grouped by PA tech

% Load plot data for optimal P_TOT
data_plot = opt_32_c3_los75;
for makefig = 1
figure; hold on;
for tech_idx = 1:3                        % tech
    for arch_idx = 1:5                    % arch
    u_idx = size(data_plot,3);
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);     % users
    dat = squeeze(dat);
    plot(uxx(1:u_idx), dat/1e3, 'LineStyle',linVec_plot{tech_idx},'Color',clrs(arch_idx,:),...
        'LineWidth',0.8,'Marker',markerVec{tech_idx},'MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:));
    end
end
ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
xticks(1:1:u_idx); xticklabels(num2str(N_user_range(1:u_idx)'));
ylabel('P_{TOT} [W]'); xlabel('Number of users');
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"]+[" CMOS"; " GaAs"; " GaN"];
legend(reshape(legText', 1, []), 'Location','southoutside', 'NumColumns', 3);
grid on; box on;
end
title('P_{TOT} at optimum point, for EIRP = 32.5dBm, Case 3, LOS 75%');

% Load plot data for optimal array size
data_plot = loc_32_c3_los75;
for makefig = 1
% relative horizontal spacing - manual for now
% nuser x arch
tech1 = [    0     1     2     1     0;
             0     1     2     1     0;
             0     1     2     3     0];
tech2 = [    0     0     2     1     0;
             0     0     2     1     0;
             0     1     2     3     0];
tech3 = [    0     0     2     1     0;
             0     0     2     1     0;
             0     1     2     3     0];

sph = {tech1' tech2' tech3'};
% relative vertical spacing
% arch x Ntx x users
sp = zeros([size(data_plot,1) length(Ntx_range) size(data_plot,3)]);
for u_idx = 1:size(sp,3)                % num. users value
    for archs = 1:size(sp,1)         % arch type
            for nn = 1:size(sp,2)    % Ntx value
            sp(archs,nn,u_idx) = nnz(data_plot(archs,:,u_idx) == nn);
            end
    end
end

dyy = [];

figure; hold on;
for tech_idx = 1:3                        % techs
    for arch_idx = 1:size(sp,1)           % archs
    u_idx = size(sp,3);             % choose users
    dat = data_plot(arch_idx, tech_idx, 1:u_idx);    % get data as 2D mat
    dat = squeeze(dat);
    
    % for equidistant scatter points - vertical
    for ii = 1:u_idx
    if ~isnan(dat(ii))
        if sp(arch_idx,dat(ii),ii) == 3                       % tech 1,2 and 3
            dyy(ii,:) = -[-0.2 0 0.2];
        elseif sp(arch_idx,dat(ii),ii) == 2
            if loc_40_c1(arch_idx,1,ii) == loc_40_c1(arch_idx,2,ii) % tech 1 and 2
                dyy(ii,:) = -[-0.2 0.2 0]./2;
            elseif loc_40_c1(arch_idx,1,ii) == loc_40_c1(arch_idx,3,ii) % tech 1 and 3
                dyy(ii,:) = [-0.2 0 0.2]./2;
            else                                            % tech 2 and 3
                dyy(ii,:) = -[0 -0.2 0.2]./2;               
            end
        else
            dyy(ii,:) = [0 0 0];                        % 0 or 1 tech
        end
    end
    end
    
    % for equidistant scatter points - horizontal
    dxx = sph{tech_idx}(arch_idx,:).*0.1;
     
    xoff = 0.2;
   
    scatter(uxx(1:u_idx)+xoff+dxx, dat + dyy(:,tech_idx), markerSz{tech_idx}, clrs(arch_idx,:), 'LineWidth', 0.8, ...
        'Marker',markerVec{tech_idx},'MarkerFaceColor',markerFaceClr{tech_idx}(arch_idx,:),'MarkerEdgeColor',clrs(arch_idx,:));
    end
end
ax = gca; ax.GridLineStyle = '-.'; ax.GridColor = [0.2510    0.2510    0.2510];
xticks(1:1:u_idx); 
xticklabels(num2str(N_user_range(1:u_idx)')); xlim([0.5, u_idx(end)+1]);
% %xticks = [];
yticks(0:1:6); 
yticklabels(['  0'; num2str(Ntx_range');'520']); 
ylim([0 6.5]);
ylabel('N_{TX}'); xlabel('Number of users');
legText = ["MBPAA","DMBA","HFC","SDMBA","HSA"]+[" CMOS"; " GaAs"; " GaN"];
legend(reshape(legText', 1, []), 'Location','southoutside', 'NumColumns', 3);
grid on;
box on;
set(gca,'YMinorTick','on');
ax.YAxis.MinorTickValues = [0.5:1:6.5]; %grid minor;
%ax.YRuler.MinorTick = [0.1:0.1:0.9];
end
title('Array size at optimum point, for EIRP = 32.5dBm, Case 3, LOS 75%');