function [] = plotGroupedBarsPdc(arch_name, case_num, arch_P_tot_cell, arch_P_tot_breakdown_cell, choose_configs_plot, K_range, M_range, comp_names, N_user)
% Author: Natasha Birari, natashabirari@gmail.com, Aug 2022
%
% This function plots the system power consumption (including component breakdown)
% of sub-array type architectures, with varying total number of transmit
% antennas and for different types of sub-array configurations. 
%
% For the sub-array architectures in this model, the main parameter of
% interest is the number of antennas per sub-array, 'K'. The various values
% of 'K' are the differentiators between the various sub-array configurations.
% For each bar, the number of antennas per sub-array 'K' and the associated number 
% of RF chains 'M' is indicated.
%
% NOTE: For SDMBA, this function leaves out K = 32 due to too high Pdc. This
% can be manually changed. 
%
% INPUTS:
% arch_name:        name of archicture (for the plot title)
% arch_P_tot_cell:  mW, total power of that architecture for all sub-array 
% (data)            configs, arranged in a cell array with each cell
%                   representing one config.
% arch_P_tot_breakdown_cell: power breakdown of that architecture for all
% (data)            sub-array configs, arranged in a cell array with each
%                   cell representing one config.
% choose_configs_plot: number of configs to plot, in order of appearance in
%                   K_range
% K_range:          row vector of values of K (antennas per sub-array)
% M_range:          row vector of values of M (associated RF chains), same
%                   size as K_range
% comp_names:       component names for legend, string array
% N_user:           number of users (for plot title, since the N_user
%                   is already incorporated into the input P_dc data)
%
% OUTPUT:
% output_plot: figure

    % coordinates for text labels
    txtoffset = -8;
    txtoffset2 = 3;

    if isstring(choose_configs_plot) == 1
        choose_configs_plot = [1:numel(K_range)];
    end
    
    if arch_name == "SDMBA"
        % Do not plot the case of K = 32 for SDMBA because the power consumption was too
        % high --> set as NaN.
        arch_P_tot_cell{K_range == 32}(:) = NaN;
        arch_P_tot_breakdown_cell{K_range == 32}(:) = NaN;
        choose_configs_plot(K_range == 32) = [];
        txtoffset = -30;
        txtoffset2 = 10;
    end
    
    %% set up plot's supporting info and properties
    
    % Define x axis for grouped bars
    xx = [1:length(arch_P_tot_cell{1})].*(max(choose_configs_plot)+1);
    
    % text under x axis (num. subarray elements K, num. RF chains M)
    K_range_str = "K = "+K_range;
    %K_range_str2 = ["2","4","8","16","32"];
    K_range_str2 = string(K_range);
    subarrtext_K = repmat(K_range_str,length(xx),1);
    subarrtext_K2 = repmat(K_range_str2,length(xx),1);
    subarrtext_M = "M = "+M_range';
    subarrtext_M2 = string(M_range');
    
    % Colours and markers
    %linecolors = ['k','g','b','m','c','r']; % upto 6 configurations can be plotted
    linecolors = ['k','k','k','k','k','k'];  % all black
    markerVec = ['o','*','d','^','s','x']; % six markers
    barcolors = colormap(flipud(parula(length(comp_names{1}))));        % based on number of components being plotted
    %barcolors(end,:) = [0.9 0 0.1];    % for jet
    
    %% Plot
    output_plot = figure; hold on; 
    % loop through the subarray configurations, use different for-loops to preserve order of visibility
    % plot breakdown power
    for i = choose_configs_plot
        bp = bar(xx+i-1,arch_P_tot_breakdown_cell{i}'/1000, 0.15, 'stacked', 'FaceColor', "flat");
        % use the same colors for stacked bar
       for j = 1:length(bp)
            set(bp(j), 'FaceColor', barcolors(j,:));
       end
    end
    legend(comp_names{1}(1:end), 'Location','NW'); 
    % Plot total power
    for i = choose_configs_plot
        %plot(xx+i-1, arch_P_tot_cell{i}/1000,'-','Color', linecolors(i), 'Marker', markerVec(i), 'LineWidth', 1, 'DisplayName',"Total,"+K_range_str(i));
%         ORIGINAL
%         indicate K and M values below bars
%         text(xx+i-1,arch_P_tot_cell{i}/1000+4,subarrtext_K(:,i),'vert','bottom','horiz','center'); 
        for q = 1:length(arch_P_tot_cell{i})
            if ~isnan(arch_P_tot_cell{i}(q))    % only if bar is plotted
                text(xx(q)+i-1,txtoffset,subarrtext_K2(q,i),...
                    'vert','bottom','horiz','center',...
                    'FontSize',8); 
                text(xx(q)+i-1,txtoffset-txtoffset2,subarrtext_M2(q,i),...
                    'vert','bottom','horiz','center',...
                    'FontSize',8); 
            end
        end
    end
    hold off;
    %title({"Power consumption breakdown for "+arch_name,"Case "+case_num+", "+N_user+" users"});
    title(arch_name);
    % ax.TickLabelInterpreter = 'tex';
    xx2 = [0,xx];
    xticks(xx2+1);
%    xticklabels({'Total number of antennas, Ntx','16','32','64','128','256','384','512'});
    xticklabels({'Ntx','16','32','64','128','256','512'}); 
    text(xx2(1)+1,txtoffset,"K",'vert','bottom','horiz','right'); 
    text(xx2(1)+1,txtoffset-txtoffset2,"M",'vert','bottom','horiz','right'); 
    ylabel("Total Power Comsumption[W]"); 
    % xlabel('N_{TX}');
    grid on; box on;
end